#ifndef MVIMPACT_BLOB_H_
#define MVIMPACT_BLOB_H_


/// \file
///

#ifndef WRAP_PYTHON
#include "mvIMPACT.h"
#endif // #ifndef WRAP_PYTHON


#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_PYTHON


#include <vl_blob.h>
#include <vl_blobloc.h>


#endif // #ifndef WRAP_PYTHON
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

namespace mvIMPACT {

        /// \namespace mvIMPACT::blob This namespace contains all classes and functions from mvIMPACT Blob.
	namespace blob {


/// \brief Special value for threshold
///
/// \see mvIMPACT::blob::BlobFeatureList::getThreshold
/// \see mvIMPACT::blob::BlobFeatureList::setThreshold
/// \see mvIMPACT::blob::BlobLocator::getThreshold
/// \see mvIMPACT::blob::BlobLocator::setThreshold
enum BlobThresholdMode
{
	/// Use Otsu's method for determining a threshold automatically 
	btmAuto = IPL_DONT_CARE,
};

typedef int BlobFeatureID; 


#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY

/// "ONLY USED INTERNALLY"
class BlobResult 
{
public:
	// default values are needed because std::map needs a standard constructor
	// for its value type.
	BlobResult(
		mvIMPACT_C::BL_FEATURE feature=(mvIMPACT_C::BL_FEATURE)0, 
		mvIMPACT_C::IPL_ULONG subItemIndex=(mvIMPACT_C::IPL_ULONG)-1);

	
	mvIMPACT_C::BL_FEATURE getFeature() const;
	mvIMPACT_C::IPL_ULONG getSubItemIndex() const;
	
	
private:
	mvIMPACT_C::BL_FEATURE m_feature;
	mvIMPACT_C::IPL_ULONG m_subItemIndex;
};

class BlobFeatureList;

/// "ONLY USED INTERNALLY"
/// Base class for wrapping of single blob features
class BlobFeature
{
public:
	BlobFeature(
		BlobFeatureList* blobFeatureList,
		mvIMPACT_C::BL_FEATURE blFeature,
		void* param,
		int paramSize,
		int id
		);

	void remove();
	
	/// returns the BlobResult structure which corresponds to the feature
	/// when retrieving this result from the C-API's result buffer
	mvIMPACT_C::BL_FEATURE getBlFeature() const;

	BlobFeatureID getId() const;				
		
private:
	BlobFeatureList* m_blobFeatureList;
	mvIMPACT_C::BL_FEATURE m_blFeature;
	BlobFeatureID m_Id;
	DynArray<char> m_Param; // raw arameter data, NULL if there isn't any
};


#endif // #ifndef WRAP_ANY
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS


/// \brief Represents a single run of a blob. 
///
///It consists of three coordinates: The y coordinate of
/// the scanline and the two x coordinates for the start (x0) and the end (x1) of the run.
class RunLine
{
public:
	/// \brief construct a RunLine object from the three coordinates
	///
	 // \internal
	 // If you change the default initializations, you have to 
	 // change them in base.i as well in the tuple to class conversion !
	RunLine(
		/// first x coordinate
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			   
		int x0=0,
		/// second x coordinate
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			   
		int x1=0,
		/// the y coordinate
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			   
		int y=0);
	
	/// return the y coordinate
	int getY() const;
	
#ifndef WRAP_ANY
	/// return the y coordinate
	int & getY();
#endif // #ifndef WRAP_ANY

	/// set the y coordinate
	RunLine& setY(
		///new y coordinate value
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			   
		int y);
	
	/// return the x0 coordinate, i.e. the coordinate of the start of the run.
	int getX0() const;
	
#ifndef WRAP_ANY
	/// return the x0 coordinate, i.e. the coordinate of the start of the run.
	int & getX0();
#endif // #ifndef WRAP_ANY

	/// set the x0 coordinate, i.e. the coordinate of the start of the run.
	RunLine& setX0(
		///new value of the first x coordinate
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			   
		int x0);
	
	/// return the x1 coordinate, i.e. the coordinate of the end of the run.
	int getX1() const;

#ifndef WRAP_ANY
	/// return the x1 coordinate, i.e. the coordinate of the end of the run.
	int & getX1();
#endif // #ifndef WRAP_ANY

	/// set the x1 coordinate, i.e. the coordinate of the end of the run.
	RunLine& setX1(
		///new value of the second x coordinate
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			   
		int x1);

private:
		int m_y;
		int m_x0;
		int m_x1;
};


class BlobResultList;
 
/// \brief This class holds all features that should be searched for during the analysis
/// 
/// You must add all the features to search for to this class before calling mvIMPACT::blob::analyze.
/// Furthermore you can use this class to adjust some of the analysis parameters.
//-----------------------------------------------------------------------------
class BlobFeatureList
//-----------------------------------------------------------------------------
{
private:
	/// Unassigned buffer constant.
	enum { unassigned = -1 };
	/// The buffer handle 
	mvIMPACT_C::BL_BUFHANDLE m_handle;
	/// Pointer to the reference counter.
	int* m_counter;
	
	/// Pointer to the id counter (this indirection is for correct copy semantics of the class)
	int *m_idCounter;
	
	/// map from C-API feature identifier to list of equal features with different parameters
	/// (this indirection is for correct copy semantics of the class)
	std::map< mvIMPACT_C::BL_FEATURE, std::list< BlobFeature > > *m_FeatureMap;

	/// map from ID to iterator in the corresponding sublist of m_FeatureMap
	/// (this indirection is for correct copy semantics of the class)
	std::map< BlobFeatureID, std::list< BlobFeature >::iterator > *m_IdMap;
	
public:

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
	mvIMPACT_C::BL_BUFHANDLE getHandle() const;
	std::map< mvIMPACT_C::BL_FEATURE, std::list< BlobFeature > > & getFeatureMap();
	std::map< BlobFeatureID, std::list< BlobFeature >::iterator > & getIdMap();
#endif // #ifndef WRAP_PYTHON
#endif //DOXYGEN_SHOULD_SKIP_THIS

	
public:
	/// \brief Construct an allocated BlobFeatureList
	BlobFeatureList();

	/// \brief Copy construct a feature list(share ownership)
	BlobFeatureList(///the other list
			const BlobFeatureList& src);

#ifndef WRAP_ANY
	/// \brief Overwrite current object with \a src.
	BlobFeatureList& operator=(///teh other feature list
				   const BlobFeatureList& src);
#endif // #ifndef WRAP_PYTHON


	/// \brief Destruct a feature list
	virtual ~BlobFeatureList();
	
	///\brief Set neighborhood mode
	///
	///This specifies which neighborhood mode to use during analysis. You can choose a 4- or 8-neighborhood mode.
	/// \see #NeighborhoodMode
	BlobFeatureList& setNeighborhoodMode(///the neighborhood mode 
					     NeighborhoodMode neighborhoodMode);

	/// \brief Get neighborhood mode
	///
	///Returns the neighborhood mode
	///\see #NeighborhoodMode
	NeighborhoodMode getNeighborhoodMode() const;


	///\brief Get maximum number of indices processed
	///
	/// If there are more blobs than this value then an exception is thrown.
	unsigned int getStopAfterIndex() const;

	///\brief Set maximum number of indices processed
	///
	/// If there are more blobs than this value then an exception is thrown.
	BlobFeatureList& setStopAfterIndex(
		/// maximal number of blobs
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			   
		unsigned int stopAfterIndex);

	///\brief return if blobs, which touch the border, are removed
	bool getKillBorderObjects() const;

	///\brief set if blobs, which touch the border, are removed
	BlobFeatureList& setKillBorderObjects(bool killBorderObjects);

	///\brief Get minimum object size 
	///
	///Returns the size a blob must exceed to be considered as a valid blob.
	unsigned int getMinObjectSize() const;

	///\brief Set minimum object size 
	///
	///This specifies the size a blob must exceed to be considered a valid blob.
	BlobFeatureList& setMinObjectSize(
		///new minimal object size 
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.maxObjectSize"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		unsigned int minObjectSize);

	///\brief Get maximum object size 
	///
	///Returns the size a blob must not reach to be still considered as a valid result.
	unsigned int getMaxObjectSize() const;

	///\brief Set maximum object size
	///
	///This specifies the size a blob must not reach to be still considered as a valid result.
	BlobFeatureList& setMaxObjectSize(
		///new maximal object size
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamComplex Python="%(instance)s.minObjectSize"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			   
		unsigned int maxObjectSize);


	///\brief Get threshold
	///
	/// Returns the threshold used when calling mvIMPACT::blob::analyzeGray
	/// 
	/// If the threshold value is set to \c btmAuto, Otsu's method is used for
	/// determining a threshold automatically.
	///
	/// \see #BlobThresholdMode
	int getThreshold() const;
	
	///\brief Set threshold
	///
	///This specifies which threshold to use when calling mvIMPACT::blob::analyzeGray.
	///
	/// If the threshold value is set to \c btmAuto, Otsu's method is used for
	/// determining a threshold automatically.
	///
	/// \see #BlobThresholdMode
	BlobFeatureList& setThreshold(
		///the new threshold value
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="-1"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			   
		int threshold);

	///\brief Get polarity mode
	///
	///This specifies which polarity mode to use during analysis. 
	///You can choose \c pmWhiteOnBlack for "white on black" or \c pmBlackOnWhite for 
	/// "black on white" mode. Note that mode \c pmAuto is not supported here.
	///\see #PolarityMode
	PolarityMode getPolarityMode() const;

	///\brief Set polarity mode
	///
	///This specifies which polarity mode to use during analysis. 
	///You can choose \c pmWhiteOnBlack for "white on black" or \c pmBlackOnWhite for 
	/// "black on white" mode. Note that mode \c pmAuto is not supported here.
	///\see #PolarityMode
	BlobFeatureList& setPolarityMode(PolarityMode polarityMode);

	
	/// Returns the current coordinate transformation
	///
	/// The returned transformation is used to calibrate the results.
	/// \see mvIMPACT::Position
	Position getCoordinateSystem() const;

	/// \brief Sets the coordinate transformation
	///	
	/// This can be used to calibrate the results.
	/// \see mvIMPACT::Position
	BlobFeatureList& setCoordinateSystem(///the linear transformation to use
					     const Position& coordinateSystem);


#ifndef DOXYGEN_SHOULD_SKIP_THIS
	/// transfers the data necessary to retrieve the results from
	/// \a dest
	void transfer(BlobResultList& dest) const;
#endif //DOXYGEN_SHOULD_SKIP_THIS


	/// \brief The number of foreground pixels in a blob.
	///
	/// The number of pixels is multiplied by the scaled pixel area. 
	/// Holes will not contribute to the area.
	/// \see mvIMPACT::blob::BlobFeatureList::setCoordinateSystem
	BlobFeatureID addArea();
	
	/// \brief The total length of edges in a blob given in calibrated units. 
	///
	///Holes will not contribute to the 
	/// perimeter. A single pixel with an aspect ratio of 1.0 will have a perimeter of 4.0. It is equal to:
	///
	/// \image html images/mvi_blob/Bl_Perimeter.png
	/// \see mvIMPACT::blob::BlobFeatureList::setCoordinateSystem
	BlobFeatureID addPerimeter();
	
	/// \brief This value is derived from the perimeter and the area of an object. 
	///
	///For circlular objects, this 
	/// value ranges close to 1 and for elongated or rough objects it is close to 0. It is equal to: 
	///
	/// \image html images/mvi_blob/Bl_Roundness.png
	BlobFeatureID addRoundness();
	
	/// \brief Specifies the maximum and minimum coordinates of an object (bounding box).
	///
	/// The coordinates will be returned as two 
	/// points in calibrated units: a top left point (XMIN, YMIN) and a bottom right point (XMAX, YMAX).
	///
	/// Thus, the Feret diameters are given by: y<BR>
	/// FERETX = XMAX - XMIN + 1 <BR>
	/// FERETY = YMAX - YMIN + 1 <BR>
	///
	/// \image html images/mvi_blob/Bl_Extreme.png
	/// \see mvIMPACT::blob::BlobFeatureList::setCoordinateSystem
	BlobFeatureID addExtremeBox();

	/// \brief These coordinates refer to the first pixel of an object which is hit, when scanning an image line by line 
	/// from left to right and from top to bottom. 
	///
	/// These coordinates are returned in calibrated units.
	/// 
	/// \see mvIMPACT::BlobFeatureList::setCoordinateSystem
	BlobFeatureID addFirstPoint();
	

	/// \brief This measurement specifies the Euler number of each blob. 
	///
	///Because every blob is measured individually, 
	/// the Euler number can be defined as follows:
	///
	/// EULER = 1 - number of holes.
	///
	BlobFeatureID addEulerNumber();
	
	/// \brief This number specifies the number of holes in a blob. 
	///
	///It is derived from the EULER number and is equal to:
	///
	/// Number of holes = 1 - EULER Number
	///
	/// Holes may not intersect the edge of an image (in this case, they are not counted).
	BlobFeatureID addNumberHoles();

	/// \brief This measurement calculates the x, y coordinates of the end points of the major axis. 
	///
	///The major axis is 
	/// defined as the longest line through an object passing through the centroid. The x, y coordinates will 
	/// be given in calibrated units.
	/// Set \a grayScaleImage to \c true if you want to use the gray values, or \c false if you want to use the binary version.
	///
	/// \image html images/mvi_blob/Bl_MajorAxis.png
	/// \see mvIMPACT::blob::BlobFeatureList::setCoordinateSystem
	///
	BlobFeatureID addMajorAxis(
		/// This parameter specifies the number of steps used to find the major axis end points. The more angles you 
		/// use, the more accurate the result will be. However, it will take longer to calculate.		
		/// The default value 16 corresponds to an angle of 360/16 = 22.5 degrees.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="360"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int steps=16, 
		///If set, moments are calculated using gray values, else use binary version.
		bool grayscaleImage=false);
	


	/// \brief This measurement specifies the angle in degrees between the positive x-axis and the major axis. 
	///
	/// The angle ranges between 0..180.0 degrees and is measured in anti-clockwise direction from the positive x-axis. 
	/// If the angle is smaller than 0.000001 degrees, it will be clipped to 0.
	/// Set \a grayScaleImage to \c true if you want to use the gray values, or \c false if you want to use the binary version.
	///
	/// \image html images/mvi_blob/Bl_MajorAxis.png
	///
	BlobFeatureID addMajorAxisAngle(
					/// This parameter specifies the number of steps used to find the major axis end points. The more angles you 
					/// use, the more accurate the result will be. However, it will take longer to calculate.		
					/// The default value 16 corresponds to an angle of 360/16 = 22.5 degrees.
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
					/// <mvParamMax><mvParamConst value="360"/></mvParamMax>
					/// </mvParamRange>
					/// \endxmlonly			   
					int steps=16, 
					///If set, moments are calculated using gray values, else use binary version.
					bool grayscaleImage=false);
	

	/// \brief This measurement specifies the length in calibrated units of the major axis. 
	///
	///The length is calculated 
	/// using the Euclidean distance. It is calculated as follows:
	/// 
	/// \image html images/mvi_blob/Bl_MajorAxisLength.png
	/// Set \a grayScaleImage to \c true if you want to use the gray values, or \c false if you want to use the binary version.
	/// \see mvIMPACT::blob::BlobFeatureList::setCoordinateSystem
	///
	BlobFeatureID addMajorAxisLength(
					/// This parameter specifies the number of steps used to find the major axis end points. The more angles you 
					/// use, the more accurate the result will be. However, it will take longer to calculate.		
					/// The default value 16 corresponds to an angle of 360/16 = 22.5 degrees.
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
					/// <mvParamMax><mvParamConst value="360"/></mvParamMax>
					/// </mvParamRange>
					/// \endxmlonly			   
					int steps=16, 
					///If set, moments are calculated using gray values, else use binary version.
					bool grayscaleImage=false);
	

	/// \brief This measurement calculates the x, y coordinates of the end points of the minor axis. 
	///
	///The minor axis is 
	/// defined as a line passing through the centroid of an object while maintaining perpendicularity with the 
	/// major axis. The x, y coordinates will be given in calibrated units.
	/// Set \a grayScaleImage to \c true if you want to use the gray values, or \c false if you want to use the binary version.
	///
	/// \image html images/mvi_blob/Bl_MajorAxisLength.png
	/// \see mvIMPACT::blob::BlobFeatureList::setCoordinateSystem
	///
	BlobFeatureID addMinorAxis(
		/// This parameter specifies the number of steps used to find the major axis end points. The more angles you 
		/// use, the more accurate the result will be. However, it will take longer to calculate.		
		/// The default value 16 corresponds to an angle of 360/16 = 22.5 degrees.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="360"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int steps=16, 
		///If set, moments are calculated using gray values, else use binary version.
		bool grayscaleImage=false);

		
	/// \brief This measurement specifies the angle in degrees between the positive x-axis and the minor axis. 
	///
	///The angle 
	/// ranges between 0..180.0 degrees and is measured in anti-clockwise direction from the positive x-axis. 
	/// An angle smaller than 0.000001 degrees will be clipped to 0.
	/// Set \a grayScaleImage to \c true if you want to use the gray values, or \c false if you want to use the binary version.
	///
	BlobFeatureID addMinorAxisAngle(
		/// This parameter specifies the number of steps used to find the major axis end points. The more angles you 
		/// use, the more accurate the result will be. However, it will take longer to calculate.		
		/// The default value 16 corresponds to an angle of 360/16 = 22.5 degrees.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="360"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int steps=16, 
		///If set, moments are calculated using gray values, else use binary version.
		bool grayscaleImage=false);

	/// \brief This measurement specifies the length in calibrated units of the minor axis. 
	///
	///The length is calculated 
	/// using the Euclidean distance. It is calculated as follows:
	/// Set \a grayScaleImage to \c true if you want to use the gray values, or \c false if you want to use the binary version.
	/// 
	/// \image html images/mvi_blob/Bl_MajorAxisLength.png
	/// \see mvIMPACT::blob::BlobFeatureList::setCoordinateSystem
	///
	BlobFeatureID addMinorAxisLength(
		/// This parameter specifies the number of steps used to find the major axis end points. The more angles you 
		/// use, the more accurate the result will be. However, it will take longer to calculate.		#
		/// The default value 16 corresponds to an angle of 360/16 = 22.5 degrees.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="360"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int steps=16, 
		///If set, moments are calculated using gray values, else use binary version.
		bool grayscaleImage=false);



	/// \brief This measurement determines the maximum Feret diameter of an object. 
	///
	///It is obtained, by checking a 
	/// certain number of angles. The more angles you use, the more accurate the result will be. However, it 
	/// will take longer to calculate. The maximum diameter will be returned in calibrated units.
	///
	/// \image html images/mvi_blob/Bl_MaxFeret.png
	/// \see mvIMPACT::blob::BlobFeatureList::setCoordinateSystem
	///
	BlobFeatureID addFeretMax(
		/// This parameter specifies the number of angles checked used to find the maximum Feret diameter.
		/// The default value 8 corresponds to an angle of 180/8 = 22.5 degrees.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="180"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int steps=8 
	);

	/// \brief This measurement specifies the angle at which the maximum Feret diameter was found. 
	///
	///The angle will 
	/// be returned in degrees measured from the positive x-axis in anti-clockwise direction.
	///
	/// \image html images/mvi_blob/Bl_MaxFeret.png
	///
	BlobFeatureID addFeretMaxAngle(
		/// This parameter specifies the number of angles checked used to find the maximum Feret diameter.
		/// The default value 8 corresponds to an angle of 180/8 = 22.5 degrees.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="180"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int steps=8 
	);

	/// \brief This measurement determines the minimum Feret diameter of an object. 
	///
	///It is obtained, by checking a 
	/// certain number of angles. The more angles you use, the more accurate the result will be. However, it 
	/// will take longer to calculate. The minimum diameter will be returned in calibrated units.
	///
	/// \image html images/mvi_blob/Bl_MinFeret.png
	/// \see mvIMPACT::blob::BlobFeatureList::setCoordinateSystem
	///
	BlobFeatureID addFeretMin(
		/// This parameter specifies the number of angles checked used to find the maximum Feret diameter.
		/// The default value 8 corresponds to an angle of 180/8 = 22.5 degrees.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="180"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int steps=8 
	);

	
	/// \brief This measurement determines the angle, at which the minimum Feret diameter was found. 
	///
	///The angle will 
	/// be returned in degrees measured from the positive x-axis in anti-clockwise direction.
	///
	/// \image html images/mvi_blob/Bl_MinFeret.png
	///
	BlobFeatureID addFeretMinAngle(
		/// This parameter specifies the number of angles checked used to find the maximum Feret diameter.
		/// The default value 8 corresponds to an angle of 180/8 = 22.5 degrees.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="180"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int steps=8 
	);


	/// \brief This measurement calculates the Feret diameter in calibrated units at a specified angle given in degrees. 
	///
	/// The angle is measured from the positive x-axis in anti-clockwise direction. The range for the angle 
	/// is between 0..180.0 degrees.
	/// Note that for values of 0 and 90 optimized versions of the algorithm are used.
	/// 
	/// \image html images/mvi_blob/Bl_Feret_Gen.png
	/// \see mvIMPACT::blob::BlobFeatureList::setCoordinateSystem
	///
	BlobFeatureID addFeretDiameter(
		///the angle in degrees
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="180.0"/></mvParamMax>
		/// <mvParamStep>0.1</mvParamStep>
		/// </mvParamRange>
		/// \endxmlonly			   
		double angle);

	/// \brief Aspect ratio of an object using the maximum and minimum Feret diameter.
	///
	/// It is equal to: 
	///
	/// FeretRatio = FeretMax / FeretMin
	///
	BlobFeatureID addFeretRatio(
		/// This parameter specifies the number of angles checked used to find the maximum Feret diameter.
		/// The default value 8 corresponds to an angle of 180/8 = 22.5 degrees.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="180"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int steps=8);
	
	
	/// \brief The returned value specifies an approximation of the convex perimeter of an object. 
	///
	///It is obtained by 
	/// calculating several Feret diameters at different angles. The more angles you use, the more accurate 
	/// the result will be. However, it will take longer to calculate. The convex perimeter is the sum of all 
	/// Euclidean distances between points on the convex hull.     
	///
	/// \image html images/mvi_blob/Bl_Perimeter_Convex.png
	///
	BlobFeatureID addPerimeterConvex(
		/// This parameter specifies the number of angles checked used to find the maximum Feret diameter.
		/// The default value 8 corresponds to an angle of 180/8 = 22.5 degrees.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="180"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int steps=8 
	);

	/// \brief Describes the surface roughness of an object. 
	///
	///A smooth convex object will have a roughness close to 1.0.
	/// It is equal to:
	///
	/// Roughness = Perimeter / Convex perimeter
	/// 
	///
	BlobFeatureID addRoughness(
		/// Specifies the number of steps used during the rotation.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="180"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int steps=8 
	);


	/// \brief The returned value specifies an approximation of the convex area of an object. 
	///
	///It is obtained by 
	/// calculating several Feret diameters at different angles. The more angles you use, the more accurate 
	/// the result will be. However, it will take longer to calculate. Holes will not contribute to the convex area.	
	/// 
	/// \image html images/mvi_blob/Bl_AreaConvex.png
	///
	BlobFeatureID addAreaConvex(
		/// Specifies the number of steps used during the rotation.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="180"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int steps=8
	);

	/// \brief The length of each horizontal scan line within a binary object will be accumulated. 
	///
	///The returned 
	/// value will be given in pixel units (no calibration).
	/// 
	/// \image html images/mvi_blob/Bl_AreaConvex.png
	///
	BlobFeatureID addChordSum();
					
	/// \brief This is the number of pixels, that belong to a hole in a blob. 
	///
	///If there is more than one hole in an 
	/// object, the returned value is the sum of all hole pixels. The number of pixels is multiplied by 
	/// the scaled pixel area.
	///
	BlobFeatureID addAreaHoles();

	/// \brief This is a measure of the perforation of an object. 
	///
	///It is equal to:
	///
	/// Mean hole area = hole area / area
	///
	BlobFeatureID addAreaPerforation();


	/// \brief This is the mean hole area of holes in a blob. 
	///
	///It is equal to:
	///
	/// Mean hole area = hole area / number of holes
	///
	BlobFeatureID addAreaHolesMean();
		
	/// \brief The fibre measurements calculate various fibre parameters. 
	///
	///This measurement treats objects 
	/// as fibres (i.e. fibres are very long and thin with regard to their width). The measurement thus 
	/// make sense for fibre type objects only. Measures the length of a fibre in calibrated units.
	/// \see mvIMPACT::blob::BlobFeatureList::setCoordinateSystem
	///
	BlobFeatureID addFiberLength();
	
	/// \brief The fibre measurements calculate various fibre parameters. 
	///
	///This measurement treats objects as 
	/// fibres (i.e. fibres are very long and thin with regard to their width). The measurement thus make 
	/// sense for fibre type objects only. Measures the average width of a fibre in calibrated units. 
	/// The average width along the fibre is returned.
	/// \see mvIMPACT::blob::BlobFeatureList::setCoordinateSystem
	///
	BlobFeatureID addFiberWidth();

	/// \brief Calculates points of the convex hull. It is obtained by calculating several Feret diameters at different 
	/// angles. 
	///
	///The more angles you use, the more accurate the result will be. However, it will take longer to 
	/// calculate.      
	///
	BlobFeatureID addConvexhullPoints(
		/// Specifies the number of steps used during the rotation.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="180"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int steps=8);

	/// \brief This measurement represents the minimum pixel intensity value found in an object. 
	///
	/// A grayscale image is required, i.e. analyze(const Image&, const Image&, Image&, const BlobFeatureList&)
	/// must be called.
	///
	BlobFeatureID addMin();

	/// \brief This measurement represents the maximum pixel intensity value found in an object. 
	///
	/// A grayscale image is required, i.e. analyze(const Image&, const Image&, Image&, const BlobFeatureList&)
	/// must be called.
	///
	BlobFeatureID addMax();

	/// \brief This measurement represents the sum of all pixel intensity values found in an object. 
	///
	/// A grayscale image is required, i.e. analyze(const Image&, const Image&, Image&, const BlobFeatureList&)
	/// must be called.
	///
	BlobFeatureID addSum();

	/// \brief This measurement represents the squared sum of all pixel intensity values found in an object. 
	///
	/// A grayscale image is required, i.e. analyze(const Image&, const Image&, Image&, const BlobFeatureList&)
	/// must be called.
	///
	BlobFeatureID addSquaredSum();

	/// \brief This measurement represents the mean intensity value of an object. 
	///
	/// A grayscale image is required, i.e. analyze(const Image&, const Image&, Image&, const BlobFeatureList&)
	/// must be called.
	///
	BlobFeatureID addMean();
	
	/// \brief This measurement represents the standard deviation of pixel intensity values in an object. 
	///
	/// It will be derived from the object histogram.
	///
	/// \image html images/mvi_blob/Bl_StdDev.png
	///
	/// A grayscale image is required, i.e. analyze(const Image&, const Image&, Image&, const BlobFeatureList&)
	/// must be called.
	///
	BlobFeatureID addStddev();

	/// \brief This value represents the skewness of an object histogram. 
	///
	///It is also called third order central moment 
	/// of the histogram. It is equal to:
	///
	/// \image html images/mvi_blob/Bl_Skewness.png
	///
	/// A grayscale image is required, i.e. analyze(const Image&, const Image&, Image&, const BlobFeatureList&)
	/// must be called.
	///
	BlobFeatureID addSkewness();


	/// \brief This value represents the kurtosis of an object histogram. 
	///
	///It is a measure, of the peakiness of the 
	/// pixel intensity values in an area (also called forth order central moment of the histogram).
	/// It is equal to:
	///
	/// \image html images/mvi_blob/Bl_Kurtosis.png
	///
	/// A grayscale image is required, i.e. analyze(const Image&, const Image&, Image&, const BlobFeatureList&)
	/// must be called.
	///
	BlobFeatureID addKurtosis();
	
	/// \brief This is a value without a dimension representing a skewness factor. 
	///
	///A positive value represents a left 
	/// oriented distribution while a negative value represents a right oriented distribution.
	/// It is equal to:
	///
	/// \image html images/mvi_blob/Bl_SkewnessFac.png
	///
	/// A grayscale image is required, i.e. analyze(const Image&, const Image&, Image&, const BlobFeatureList&)
	/// must be called.
	///
	BlobFeatureID addSkewnessFac();

	/// \brief This is a value without a dimension representing a kurtosis factor. 
	///
	///It is equal to:
	///
	/// \image html images/mvi_blob/Bl_KurtosisFac.png
	///
	/// A grayscale image is required, i.e. analyze(const Image&, const Image&, Image&, const BlobFeatureList&)
	/// must be called.
	///
	BlobFeatureID addKurtosisFac();
	
	/// \brief This is a measure of the information contained in an object. 
	///
	///It describes how many bits can be used to 
	/// describe an object. It is equal to:
	///
	/// \image html images/mvi_blob/Bl_Entropy.png
	///
	/// A grayscale image is required, i.e. analyze(const Image&, const Image&, Image&, const BlobFeatureList&)
	/// must be called.
	///
	BlobFeatureID addEntropy();
	
	/// It is equal to:
	///
	/// \image html Bl_VarCoff.png
	///
	/// A grayscale image is required, i.e. analyze(const Image&, const Image&, Image&, const BlobFeatureList&)
	/// must be called.
	///
	BlobFeatureID addVariationCoefficient();
	
	
	
	/// \brief This measurement returns a vector, which contains the pixel intensity distribution of an object.
	///
	/// For a 8 bit image it is  a vector of 256 elements.
	/// For a 16 bit image it is a vector of 65536 elements.
	/// The data type of each entry in the histogram is IPL_DTULONG.
	///
	/// A grayscale image is required, i.e. analyze(const Image&, const Image&, Image&, const BlobFeatureList&)
	/// must be called.
	///
	BlobFeatureID addHistogram();

	/// \brief This measurement calculates the cooccurrence matrix and returns a vector containing the values 
	/// of the cooccurrence matrix for an object. 
	///
	///The size of the MxM matrix depends on the number of gray 
	/// levels used (i.e. 64 graylevels produce a matrix of size 64 x 64). This can be adjusted with the 
	/// Item parameter.
	///
	/// A grayscale image is required, i.e. analyze(const Image&, const Image&, Image&, const BlobFeatureList&)
	/// must be called.
	///
	/// The parameters \a dx, \a dy and \a size define the method, how to generate the cooccurrence matrix. 
	/// It is calculated using a pixel pair (p1, p2) with a defined distance of p2(x+dx, y+dy) relative to p1(x, y).
	///
	/// \image html images/mvi_blob/Bl_COMTX.png
	///
	BlobFeatureID addCooccurrenceMatrix(
		/// delta x 
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			   
		int dx=1,
		/// delta y
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			   
		int dy=1,
		///size of the matrix in x and y direction
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			   
		int size=64);
	
	
	/// \brief The energy (also called 'angular second moment') of the object will be calculated using the 
	/// cooccurrence matrix. It is defined as follows:
	///
	/// \image html images/mvi_blob/Bl_Energy.png
	///
	/// A grayscale image is required, i.e. analyze(const Image&, const Image&, Image&, const BlobFeatureList&)
	/// must be called.
	///
	/// The parameters \a dx, \a dy and \a size define the method, how to generate the cooccurrence matrix. 
	/// It is calculated using a pixel pair (p1, p2) with a defined distance of p2(x+dx, y+dy) relative to p1(x, y).
	///
	/// \image html images/mvi_blob/Bl_COMTX.png
	///
	BlobFeatureID addEnergy(
		/// delta x 
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			   
		int dx=1,
		/// delta y
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			   
		int dy=1,
		/// size of the matrix in x and y direction
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			   
		int size=64);
	
	
	/// \brief The contrast of the object will be calculated using the cooccurrence matrix. It is defined as follows:
	///
	/// \image html images/mvi_blob/Bl_Contrast.png
	///
	/// A grayscale image is required, i.e. analyze(const Image&, const Image&, Image&, const BlobFeatureList&)
	/// must be called.
	///
	/// The parameters \a dx, \a dy and \a size define the method, how to generate the cooccurrence matrix. 
	/// It is calculated using a pixel pair (p1, p2) with a defined distance of p2(x+dx, y+dy) relative to p1(x, y).
	///
	/// \image html images/mvi_blob/Bl_COMTX.png
	///
	BlobFeatureID addContrast(
		/// delta x 
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			   
		int dx=1,
		/// delta y
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			   
		int dy=1,
		/// size of the matrix in x and y direction
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			   
		int size=64);

	/// \brief The entropy of the object will be calculated using the cooccurrence matrix. It is defined as follows:
	///
	/// \image html images/mvi_blob/Bl_EntropyCCMTX.png
	///
	/// A grayscale image is required, i.e. analyze(const Image&, const Image&, Image&, const BlobFeatureList&)
	/// must be called.
	///
	/// The parameters \a dx, \a dy and \a size define the method, how to generate the cooccurrence matrix. 
	/// It is calculated using a pixel pair (p1, p2) with a defined distance of p2(x+dx, y+dy) relative to p1(x, y).
	///
	/// \image html images/mvi_blob/Bl_COMTX.png
	///
	BlobFeatureID addEntropyCooccurrenceMatrix(
		/// delta x 
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			   
		int dx=1,
		/// delta y
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			   
		int dy=1,
		/// size of the matrix in x and y direction
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			   
		int size=64);
	
	
	/// \brief Returns the x-position of the center of gravity of an object. 
	///
	///The coordinates will be returned in 
	/// calibrated units.
	///
	/// \image html images/mvi_blob/Bl_CenterX.png
	/// Set \a grayScaleImage to \c true if you want to use the gray values, or \c false if you want to use the binary version.
	/// \see mvIMPACT::blob::BlobFeatureList::setCoordinateSystem
	///
	BlobFeatureID addCenterGravityX(					
					///If set, moments are calculated using gray values, else use binary version.
					bool grayscaleImage=false);
	
	/// \brief Returns the y-position of the center of gravity of an object. 
	///
	///The coordinates will be returned in 
	/// calibrated units.
	/// Set \a grayScaleImage to \c true if you want to use the gray values, or \c false if you want to use the binary version.
	/// \see mvIMPACT::blob::BlobFeatureList::setCoordinateSystem
	///
	BlobFeatureID addCenterGravityY(
					///If set, moments are calculated using gray values, else use binary version.
					bool grayscaleImage=false);
	
	/// \brief Returns the x/y-position of the center of gravity of an object. 
	///
	///The coordinates will be returned in 
	/// calibrated units.
	/// Set \a grayScaleImage to \c true if you want to use the gray values, or \c false if you want to use the binary version.
	/// \see mvIMPACT::blob::BlobFeatureList::setCoordinateSystem
	///
	BlobFeatureID addCenterGravity(
				       ///If set, moments are calculated using gray values, else use binary version.
				       bool grayscaleImage=false);


    /// \brief Calculates central moments.
    ///
    /// Central moments are defined for a binary case, where all pixels are considered
    /// equal and for a grayscale case, where pixels are weighted by their intensity value in
    /// the grayscale image (this is slower to calculate).
    /// Central moments use coordinates that are relative to each blobs or image center of gravity and
    /// the selected coordinate system.  
    /// Central moments will be calculated as follows:<br>
    /// \image html images/mvi_blob/Bl_MomentCentral.png
    /// Generally, central moments are returned in a normalized form. 
    /// It is defined as follows:<br>
    /// \image html images/mvi_blob/Bl_MomentCentral_Normalize.png
    /// The following combinations of \a x and \a y are allowed (x,y): (0,1), (1,0), (2,0), (1,1), (0,2), (3,0),
    /// (2,1), (1,2), (0,3) and (2,2).
    /// Set \a grayScaleImage to \c true if you want to use the gray values, or \c false if you want to use the binary version.
    BlobFeatureID addMomentCentral(
		/// order of the moment (x direction: this is \c i in the formula)
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="3"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int x, 
		/// order of the moment (y direction: this is j in the formula)
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="3"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int y, 
		///If set, moments are calculated using gray values, else use binary version.
		bool grayScaleImage=false);
    

    /// \brief This is the angle of the orientation of an object. 
    ///
    ///The result will be returned in degrees and 
    /// is measured from the positive x-axis in anti clockwise direction. The range is between 0..180.0 degrees.
    /// This angle will always be calculated using the default coordinate system settings 
    /// (OrgX=0.0, OrgY=0.0, ScaleFacX=1.0, ScaleFacY=1.0, RotationAngle=0.0).
    /// It is calculated as follows:
    ///
    /// \image html Bl_Symmetry.png
    /// Set \a grayScaleImage to \c true if you want to use the gray values, or \c false if you want to use the binary version.
    ///
    BlobFeatureID addAxisSymmetryAngle(
				       ///If set, moments are calculated using gray values, else use binary version.
				       bool grayScaleImage=false);
	

    /// \brief This is the angle perpendicular to the Symmetry angle. 
    ///
    ///The result will be returned in degrees and 
    /// is measured from the positive x-axis in anti clockwise direction. The range is between 0..180.0 degrees. 
    /// This angle will always be calculated using the default coordinate system settings 
    /// (OrgX=0.0, OrgY=0.0, ScaleFacX=1.0, ScaleFacY=1.0, RotationAngle=0.0).
    BlobFeatureID addAxisSymmetryAngle90(
					 ///If set, moments are calculated using gray values, else use binary version.
					 bool grayScaleImage=false);
    
    /// \brief This measurement calculates the maximum distance from the centroid to a boundary point. 
    ///
    /// The distance will be returned in calibrated units. A value of BL_DONT_CARE in the result buffer indicates,
	/// that the feature could not be calculated.
    ///
    /// \image html images/mvi_blob/Bl_PolarDistMax.png
    /// Set \a grayScaleImage to \c true if you want to use the gray values, or \c false if you want to use the binary version.
    /// \see mvIMPACT::blob::BlobFeatureList::setCoordinateSystem
    ///
    BlobFeatureID addPolarDistMax(
				      ///If set, moments are calculated using gray values, else use binary version.
				  bool grayScaleImage=false);
    
    /// \brief This measurement calculates the minimum distance from the centroid to a boundary point. 
    ///
    /// The distance will be returned in calibrated units. A value of BL_DONT_CARE in the result buffer indicates,
	/// that the feature could not be calculated.
    ///
    /// \image html images/mvi_blob/Bl_PolarDistMax.png
    /// Set \a grayScaleImage to \c true if you want to use the gray values, or \c false if you want to use the binary version.
    /// \see mvIMPACT::blob::BlobFeatureList::setCoordinateSystem
    ///
    BlobFeatureID addPolarDistMin(
				  ///If set, moments are calculated using gray values, else use binary version.
				  bool grayScaleImage=false);
	
    /// \brief This measurement calculates distances from the centroid to the boundary points spaced at equal angles. 
    ///
    /// The distances will be returned in calibrated units in clockwise direction. The first distance in the result 
	/// buffer represents the distance to the left most boundary point. A value of BL_DONT_CARE in the result buffer indicates,
	/// that the distance at a given angle could not be calculated.
    ///
    /// \image html images/mvi_blob/Bl_PolarDist.png
    /// Set \a grayScaleImage to \c true if you want to use the gray values, or \c false if you want to use the binary version.
    /// \see mvIMPACT::blob::BlobFeatureList::setCoordinateSystem
    ///
    BlobFeatureID addPolarDist(
					/// This parameter specifies the number of steps and must be an even number. The rotation angle 
					/// increment after each step is defined as follows: Angle=360.0/number of angle steps <BR>
					/// For example, the default value 8 results in angle = 360.0/8 =45.0 degrees <BR>
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
					/// <mvParamMax><mvParamConst value="360"/></mvParamMax>
					/// </mvParamRange>
					/// \endxmlonly			   
				   int steps=8,
			       ///If set, moments are calculated using gray values, else use binary version.
			       bool grayScaleImage=false);
    
	/// \brief Calculate number of outline points
	///
	/// Calculate the number of outline points from each object.
	BlobFeatureID addNbOutlinePoints();

	/// \brief Calculate outline points
	///
	/// Calculates the coordinate (x,y) of each outline point of a single object. To obtain
	/// the results, you must use the method getSingleResult().
	/// Each point consists of two values (x,y). 
	/// The result array is organized as follows: x1,y1,x2,y2,x3,y3 aso...
	BlobFeatureID addOutlinePoints();

	/// \brief Calculate number of scanlines
	///
	/// Calculate the number of scanlines from each object.
	BlobFeatureID addNbScanlines();

	/// \brief Calculate scanlines
	///
	/// Calculates the coordinate of each scanline of a single object. To obtain
	/// the results, you must use the method getSingleResult().
	/// Each scanlines consists of three values (y,xl,xr), where
	/// y specifies the y-coordinate
	/// xl specifies the left x-coordinate and
	/// xr specifies the right x-coordinate.
	/// The result array is organized as follows: y1,xl1,xr1,y2,xl2,xr2 aso...
	BlobFeatureID addScanlines();


    /// \brief Remove a feature.
    ///
    /// To remove a feature you have to pass the mvIMPACT::blob::BlobFeatureID you got
    ///when you added the feature to this method.
    BlobFeatureList& removeFeature(///ID of the feature that should be removed
				   BlobFeatureID blobFeatureID);

	
private:
	/// dispose
	void dispose();
	
	/// copy
	void copy(const BlobFeatureList &src);
	
	/// internal convenience function
	BlobFeatureID addFeature(mvIMPACT_C::BL_FEATURE blFeature, void* param, int paramSize);

	/// construct an allocated feature from a handle
	explicit BlobFeatureList(
		/// the buffer handle
		mvIMPACT_C::BL_BUFHANDLE bufhandle);

};

///\brief This class holds the results of a blob analysis.
///
/// You can use this class to get information of the features selected
/// for the blob analysis.
class BlobResultList 
{
private:
	/// Unassigned buffer constant.
	enum { unassigned = -1 };
	/// The buffer handle 
	mvIMPACT_C::BL_BUFHANDLE m_handle;
	/// Pointer to the reference counter.
	int* m_counter;

	/// map between feature ids and results
	std::map< BlobFeatureID, BlobResult > * m_map;
	
public:
	/// \brief Construct an allocated result list
	BlobResultList();

	/// \brief Copy construct a result list (share ownership)
	BlobResultList(///the other result list
		       const BlobResultList& src);

#ifndef WRAP_ANY
	/// \brief Overwrite current object with \a src.
	BlobResultList& operator=(///the other result list
				  const BlobResultList& src);
#endif // #ifndef WRAP_ANY


	/// \brief Destruct an feature
	virtual ~BlobResultList();

	/// \brief Return the number of bands.
	int getBands() const;

	/// \brief Return the number of slices.
	int getSlices() const;
	
	/// \brief Return the number of frames.
	int getFrames() const;
	
	/// \brief Return the number of blobs for a specific plane
	///
	/// mvIMPACT Blob analyzes each plane of the image separately, thus
	///you have to select the desired plane
	int getNumberOfBlobs(
				///number of the band
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.bands-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int band=0,
			    ///number of the slice
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.slices-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int slice=0,
			    ///number of the frame
				/// \xmlonly
				/// <mvParamRange>
				/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
				/// <mvParamMax><mvParamComplex Python="%(instance)s.frames-1"/></mvParamMax>
				/// </mvParamRange>
				/// \endxmlonly			   
				int frame=0) const;
	

	/// \brief Return the number of features.
	int getNumberOfFeatures() const;
	
	/// \brief Return the number of items of a feature.
	///
	///e.g. the bounding box feature consists
	/// of 4 items.
	int getNumberOfResultItems(///ID of the desired feature 
				   BlobFeatureID blobFeatureID) const;

	/// \brief Return the datatype of a feature value.
	///
	///The datatype is expressed as a number as defined with the constants IPL_DTUCHAR,IPL_DTUSHORT ...
	int getDataType(///ID of the desired feature 
				   BlobFeatureID blobFeatureID) const;


	/// \brief Retrieve all result items of a feature and of a single blob. 
	///
	///The blob is
	/// identified by its index \a index and the feature is identified by its ID \a blobFeatureID.
	/// The \a blobFeatureID must be the value returned by the corresponding addFEATURE() function of the
	/// corresponding mvIMPACT::blob::BlobFeatureList() with which mvIMPACT::blob::analyze() was called.
	///
	/// The size of the returned vector depends on the concrete feature added.
	/// It can be determined by calling mvIMPACT::blob::BlobResultList::getNumberOfResultItems()
	///\note Some features return several items: e.g. the bounding box feature returns 4 items!
	///\see mvIMPACT::blob::BlobResultList::getNumberOfResultItems
	///\see mvIMPACT::blob::analyze
	std::vector<double> getSingleResult(///ID of the desired feature
					    BlobFeatureID blobFeatureID, 
					    ///index of the blob
						/// \xmlonly
						/// <mvParamRange>
						/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
						/// <mvParamMax><mvParamComplex Python="%(instance)s.getNumberOfBlobs(%(band)s,%(slice)s,%(frame)s)-1"/></mvParamMax>
						/// </mvParamRange>
						/// \endxmlonly			   
						int index, 
						/// number of the band
						/// \xmlonly
						/// <mvParamRange>
						/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
						/// <mvParamMax><mvParamComplex Python="%(instance)s.bands-1"/></mvParamMax>
						/// </mvParamRange>
						/// \endxmlonly			   
						int band=0, 
					    ///number of the slice		    
						/// \xmlonly
						/// <mvParamRange>
						/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
						/// <mvParamMax><mvParamComplex Python="%(instance)s.slices-1"/></mvParamMax>
						/// </mvParamRange>
						/// \endxmlonly			   
						int slice=0, 
					    ///number of the frame
						/// \xmlonly
						/// <mvParamRange>
						/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
						/// <mvParamMax><mvParamComplex Python="%(instance)s.frames-1"/></mvParamMax>
						/// </mvParamRange>
						/// \endxmlonly			   
						int frame=0) const;

	/// \brief Retrieve a single item of a feature for all blobs. 
	///
	///The item is identified
	/// by its index \a itemIndex, the feature is identified by its ID \a blobFeatureID.
	/// The \a blobFeatureID must be the value returned by the corresponding addFEATURE() function of the
	/// corresponding mvIMPACT::blob::BlobFeatureList() with which mvIMPACT::blob::analyze() was called.
	///
	/// The size of the returned vector depends on the concrete feature added.
	/// It can be determined by calling mvIMPACT::blob::BlobResultList::getNumberOfResultItems()
	///\note Some features return several items: e.g. the bounding box feature returns 4 items!
	///\see mvIMPACT::blob::BlobResultList::getNumberOfResultItems
	///\see mvIMPACT::blob::analyze
	std::vector<double> getResultItem(///ID of the feature
						BlobFeatureID blobFeatureID, 
						///the item index
						/// \xmlonly
						/// <mvParamRange>
						/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
						/// <mvParamMax><mvParamComplex Python="%(instance)s.getNumberOfResultItems(%(blobFeatureID)s)-1"/></mvParamMax>
						/// </mvParamRange>
						/// \endxmlonly			   
						int itemIndex=0, 
						///number  of the band
						/// \xmlonly
						/// <mvParamRange>
						/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
						/// <mvParamMax><mvParamComplex Python="%(instance)s.bands-1"/></mvParamMax>
						/// </mvParamRange>
						/// \endxmlonly			   
						int band=0, 
						///number of the slice
						/// \xmlonly
						/// <mvParamRange>
						/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
						/// <mvParamMax><mvParamComplex Python="%(instance)s.slices-1"/></mvParamMax>
						/// </mvParamRange>
						/// \endxmlonly			   
						int slice=0, 
						///number of the frame
						/// \xmlonly
						/// <mvParamRange>
						/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
						/// <mvParamMax><mvParamComplex Python="%(instance)s.frames-1"/></mvParamMax>
						/// </mvParamRange>
						/// \endxmlonly			   
						int frame=0) const;

	/// \brief Retrieve a single item of a feature for a single blob.
	///
	///The item is identified
	/// by its index \a itemIndex, the feature is identified by its ID \a blobFeatureID, the blob is
	/// identified by its index \a index.
	/// The \a blobFeatureID must be the value returned by the corresponding addFEATURE() function of the
	/// corresponding mvIMPACT::blob::BlobFeatureList() with which mvIMPACT::blob::analyze() was called.
	///
	/// The size of the returned vector depends on the concrete feature added.
	/// It can be determined by calling mvIMPACT::blob::BlobResultList::getNumberOfResultItems()
	///\note Some features return several items: e.g. the bounding box feature returns 4 items!
	///\see mvIMPACT::blob::BlobResultList::getNumberOfResultItems
	///\see mvIMPACT::blob::analyze
	double getSingleResultItem(///the desired feature
					BlobFeatureID blobFeatureID, 
					///the blob index
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
					/// <mvParamMax><mvParamComplex Python="%(instance)s.getNumberOfBlobs(%(band)s,%(slice)s,%(frame)s)-1"/></mvParamMax>
					/// </mvParamRange>
					/// \endxmlonly			   
					int index, 
					///the item index
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
					/// <mvParamMax><mvParamComplex Python="%(instance)s.getNumberOfResultItems(%(blobFeatureID)s)-1"/></mvParamMax>
					/// </mvParamRange>
					/// \endxmlonly			   
					int itemIndex=0, 
					///number of the band
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
					/// <mvParamMax><mvParamComplex Python="%(instance)s.bands-1"/></mvParamMax>
					/// </mvParamRange>
					/// \endxmlonly			   
					int band=0, 
					///number of the slice
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
					/// <mvParamMax><mvParamComplex Python="%(instance)s.slices-1"/></mvParamMax>
					/// </mvParamRange>
					/// \endxmlonly			   
					int slice=0, 
					///number of the frame
					/// \xmlonly
					/// <mvParamRange>
					/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
					/// <mvParamMax><mvParamComplex Python="%(instance)s.frames-1"/></mvParamMax>
					/// </mvParamRange>
					/// \endxmlonly			   
					int frame=0) const;
	
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
	mvIMPACT_C::BL_BUFHANDLE getHandle() const;

	void addFeature(BlobFeatureID id, BlobResult result) const;
#endif // #ifndef WRAP_ANY
#endif //DOXYGEN_SHOULD_SKIP_THIS

private:
	/// dispose
	void dispose();
	
	/// copy
	void copy(const BlobResultList &src);
};



/// \brief Holds configuration properties for the blob locator
///
/// This class exists for passing configuration properties to mvIMPACT::locateBlobs.
///
class BlobLocator
{
public:
	/// constructs a BlobLocator object with the following default values for the properties:
	///
	/// neighborhoodMode = nm8Connected;
	/// stopAfterIndex = 0;
	/// killBorderObjects = false;
	/// minObjectSize = 0;
	/// maxObjectSize = -1;
	/// polarityMode = pmWhiteOnBlack;
	BlobLocator();

	/// \brief Get neighborhood mode
	///
	///Returns the neighborhood mode
	///\see #NeighborhoodMode
	NeighborhoodMode getNeighborhoodMode() const;

	///\brief Set neighborhood mode
	///
	///This specifies which neighborhood mode to use during analysis. You can choose a 4- or 8-neighborhood mode.
	/// \see #NeighborhoodMode
	BlobLocator& setNeighborhoodMode(NeighborhoodMode neighborhoodMode );

	///\brief Get maximum number of indices processed
	///
	/// If there are more blobs than \a index then an exception is thrown.
	int getStopAfterIndex() const;

	///\brief Set maximum number of indices processed
	///
	/// If there are more blobs than \a index then an exception is thrown.
	BlobLocator& setStopAfterIndex(
		///maximal number of blobs
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			   
		int stopAfterIndex);

	///\brief return if blobs, which touch the border, are removed
	bool getKillBorderObjects() const;

	///\brief set if blobs, which touch the border, are removed
	BlobLocator& setKillBorderObjects(bool killBorderObjects);

	///\brief Get minimum object size 
	///
	///Returns the size a blob must exceed to be considered a valid blob.
	int getMinObjectSize() const;

	///\brief Set minimum object size 
	///
	///Specifies the size a blob must exceed to be considered a valid blob.
	BlobLocator& setMinObjectSize(
		///minimal object size
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.maxObjectSize"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int minObjectSize);

	///\brief Get maximum object size 
	///
	///Returns the size a blob must not reach to be still considered as a valid result.
	int getMaxObjectSize() const;

	///\brief Set maximum object size
	///
	///Specifies the size a blob must not reach to be still considered as a valid result.
	BlobLocator& setMaxObjectSize(
		/// maximal object size
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamComplex Python="%(instance)s.minObjectSize"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			   
		int maxObjectSize);

	///\brief Get threshold
	///
	/// Returns the threshold used when calling mvIMPACT::blob::locateBlobsGray
	///
	/// If the threshold value is set to \c btmAuto, Otsu's method is used for
	/// determining a threshold automatically.
	///
	/// \see #BlobThresholdMode
	int getThreshold() const;
	
	///\brief Set threshold
	///
	///This specifies which threshold to use when calling mvIMPACT::blob::locateBlobsGray.
	///
	/// If the threshold value is set to \c btmAuto, Otsu's method is used for
	/// determining a threshold automatically.
	///
	/// \see #BlobThresholdMode
	BlobLocator& setThreshold(
		///new threshold value
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			   
		int threshold);

	///\brief Get polarity mode
	///
	///This specifies which polarity mode to use during analysis. 
	///You can choose \c pmWhiteOnBlack for "white on black" or \c pmBlackOnWhite for 
	/// "black on white" mode. Note that mode \c pmAuto is not supported here.
	///\see #PolarityMode
	PolarityMode getPolarityMode() const;

	///\brief Set polarity mode
	///
	///This specifies which polarity mode to use during analysis. 
	///You can choose \c pmWhiteOnBlack for "white on black" or \c pmBlackOnWhite for 
	/// "black on white" mode. Note that mode \c pmAuto is not supported here.
	///\see #PolarityMode
	BlobLocator& setPolarityMode(PolarityMode polarityMode);
	
#ifndef DOXYGEN_SHOULD_SKIP_THIS
	void transferConfiguration() const;
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

private:
	NeighborhoodMode m_neighborhoodMode;
	int m_stopAfterIndex;
	bool m_killBorderObjects;
	int m_minObjectSize;
	int m_maxObjectSize;
	int m_threshold;
	PolarityMode m_polarityMode;
};

/// \brief Represents a single blob
///
/// This class can be used to refer to a single blob when
/// calling mvIMPACT::blob::analyzeSingleBlob. You can get
/// a single blob by calling mvIMPACT::blob::BlobLocatorResultList::getSingleBlob.
class SingleBlob 
{
private:
	/// Pointer to the reference counter.
	int* m_counter;
	
	/// buffer handle
	mvIMPACT_C::BLOC_BUFHANDLE m_handle;

public:
	/// creates an unallocated blob object
	SingleBlob();

#ifndef DOXYGEN_SHOULD_SKIP_THIS	
#ifndef WRAP_ANY
	explicit SingleBlob(mvIMPACT_C::BLOC_BUFHANDLE handle);
	mvIMPACT_C::BLOC_BUFHANDLE getHandle() const;
#endif // #ifndef WRAP_ANY
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS	

	/// \brief Copy construct a blob (share ownership)
	SingleBlob(const SingleBlob& src);

	/// \brief Destruct a blob
	~SingleBlob();
	
#ifndef WRAP_ANY
	/// \brief Overwrite current object with \a src.
	SingleBlob& operator=(///the other blob
				const SingleBlob& src);
#endif // #ifndef WRAP_PYTHON


private:
	void copy(const SingleBlob& src);
	
	void dispose();
};



///\brief Class that stores the results of the blob locator
class BlobLocatorResultList
{
private:
	/// Pointer to the reference counter.
	int* m_counter;
	
	/// buffer handle
	mvIMPACT_C::BLOC_BUFHANDLE m_handle;

public:
	// creates an unallocated blob locator result list
	BlobLocatorResultList();

#ifndef DOXYGEN_SHOULD_SKIP_THIS	
#ifndef WRAP_ANY
	explicit BlobLocatorResultList(mvIMPACT_C::BLOC_BUFHANDLE handle);
	mvIMPACT_C::BLOC_BUFHANDLE getHandle() const;
#endif // #ifndef WRAP_ANY
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS	

	
	/// \brief Copy construct a result list (share ownership)
	BlobLocatorResultList(///the other result list
				const BlobLocatorResultList& src);

	/// \brief Destruct a blob
	~BlobLocatorResultList();
	
#ifndef WRAP_ANY	
	/// \brief Overwrite current object with \a src.
	BlobLocatorResultList& operator=(///the other blob
				const BlobLocatorResultList& src);
#endif // #ifndef WRAP_ANY
	
	/// \brief Return the number of bands.
	int getBands() const;

	/// \brief Return the number of slices.
	int getSlices() const;

	/// \brief Return the number of frames.
	int getFrames() const;

	/// \brief Return the number of blobs for a specific plane
	///
	/// mvIMPACT Blob analyzes each plane of the image seperately, thus
	///you have to select a the desired plane
	int getNumberOfBlobs(
		/// desired band
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.bands-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int band=0,
		///desired slice
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.slices-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int slice=0,
		///desired frame
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.frames-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int frame=0) const;

	/// \brief Return a single blob
	///
	/// Return a single blob for analyzing it with mvIMPACT::blob::analyzeSingleBlob.
	SingleBlob getSingleBlob(
		/// index of the blob
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.getNumberOfBlobs(%(band)s,%(slice)s,%(frame)s)-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int index,
		/// the desired band
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.bands-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int band=0, 
		///the desired slice
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.slices-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int slice=0, 
		///the desired frame
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.frames-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int frame=0) const;

	/// \brief Return area of a single blob in a specific plane
	///
	/// Returns the area of the blob specified
	/// with \a index. 
	/// The plane in which the blob resides can be given with \a band, \a slice and \a frame
	double getArea(
		/// index of the blob
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.getNumberOfBlobs(%(band)s,%(slice)s,%(frame)s)-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int index,
		/// desired band
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.bands-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int band=0,
		/// desired slice
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.slices-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int slice=0,
		///desired frame
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.frames-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int frame=0) const;

	
	/// \brief Return the extreme box a single blob in a specific plane
	///
	/// Returns the extreme box (aka bounding box) of a blob specified with \a index.
	/// 
	/// The plane in which the blob resides can be given with \a band, \a slice and \a frame
	Rectangle getExtremeBox(
		/// index of the blob
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.getNumberOfBlobs(%(band)s,%(slice)s,%(frame)s)-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int index, 
		/// the desired band
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.bands-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int band=0,
		///the desired slice
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.slices-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int slice=0,
		/// the desired frame
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.frames-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int frame=0) const;

	/// \brief Return the outline points of a single blob in a specific plane 
	///
	/// The plane in which the blob resides can be given with \a band, \a slice and \a frame
	std::vector<Point> getOutlinePoints(
		/// index of the blob
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.getNumberOfBlobs(%(band)s,%(slice)s,%(frame)s)-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int index,
		/// the desired band
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.bands-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int band=0,
		/// the desired slice
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.slices-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int slice=0,
		/// the desired frame
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.frames-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int frame=0) const;

	/// \brief Return the run lines of a single blob in a specific plane
	///
	/// All run lines the blob consists of are returned.
	///
	/// The plane in which the blob resides can be given with \a band, \a slice and \a frame
	std::vector<RunLine> getRunLines(
		/// index of the blob
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.getNumberOfBlobs(%(band)s,%(slice)s,%(frame)s)-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int index,
		/// the desired band
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.bands-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int band=0, 
		/// the desired slice
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.slices-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int slice=0,
		/// the desired frame
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.frames-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			   
		int frame=0) const;

private:
	void copy(const BlobLocatorResultList& src);
	
	void dispose();

};


/// \brief Locate the blobs in an image
///
/// The blobs in image \a sourceDest are calculated. \a sourceDest must be a binary image. Parameters
/// on how the analyzation is done can be specified whith the configuration object
/// \a blobLocator.
///
/// The image \a sourceDest is labelled after the analyzation, i.e. all points belonging to a blob
/// are assigned the same gray value.
BlobLocatorResultList locateBlobs(/// the source image
		Image& sourceDest, 
		/// the configuration to use
		const BlobLocator &blobLocator);
		
/// \brief Locate the blobs in an image
///
/// The blobs in image \a source are calculated. \a source must be a binary image. Parameters
/// on how the analyzation is done can be specified whith the configuration object
/// \a blobLocator.
///
/// The image \a destLabel contains the labelled source image after analyzation, i.e. all points belonging to a blob
/// are assigned the same gray value.
BlobLocatorResultList locateBlobs(/// the source image
		const Image& source, 
		/// the labelled image will be stored here
		Image& destLabel,
		/// the configuration to use
		const BlobLocator &blobLocator);
		
/// \brief Locate the blobs in an grayscale image
///
/// The blobs in grayscale image \a sourceDest are calculated. \a sourceDest will be thresholded
/// before further processing. Parameters on how the analyzation is done can be 
/// specified whith the delivered configuration object
/// \a blobLocator.
///
/// The image \a sourceDest is labelled after the analyzation, i.e. all points belonging to a blob
/// are assigned the same gray value.
BlobLocatorResultList locateBlobsGray(
		/// the grayscale source image. After analyzation it contains the labelled image
		Image& sourceDest,
		/// the configuration to use
		const BlobLocator &blobLocator
		);


/// \brief Locate the blobs in an grayscale image
///
/// The blobs in grayscale image \a source are calculated. \a source will be thresholded
/// before further processing. Parameters on how the analyzation is done can be 
/// specified whith the delivered configuration object
/// \a blobLocator.
///
/// The image \a destLabel contains the labelled source image after analyzation, i.e. all points belonging to a blob
/// are assigned the same gray value.
BlobLocatorResultList locateBlobsGray(
		/// the grayscale source image
		const Image& source,
		/// the labelled source image will be stored here
		Image& destLabel,		
		/// the configuration to use
		const BlobLocator &blobLocator
		);


/// \brief Perform a blob analysis with the features provided with \a blobFeatureList.
///
/// The binary source image \a sourceDest is labelled.
///The returned BlobResultList object contains the results of the blob analysis.
/// \see mvIMPACT::blob::BlobFeatureList
//-----------------------------------------------------------------------------
BlobResultList analyze(///the source image
		       Image& sourceDest, 
		       ///the list of desired features
		       const BlobFeatureList& blobFeatureList);

/// \brief Perform a blob analysis with the features provided with \a blobFeatureList.
///
/// The binary source image is expected in \a sourceBinary, the gray values of the image is
/// expected in \a sourceGray. The labelled binary image is stored in \a destLabel.
///The returned BlobResultList object contains the results of the blob analysis.
/// \see mvIMPACT::blob::BlobFeatureList
//-----------------------------------------------------------------------------
BlobResultList analyze(///the binary source image
		       const Image& sourceBinary, 
		       ///the values of the image
		       const Image& sourceGray, 
		       ///variable that receives the labeled image
		       Image& destLabel, 
		       ///list that contains the desired features
		       const BlobFeatureList& blobFeatureList);

/*
BlobResultList analyze(/// a tracer result
				const TraceResult& source,
				/// the gray values of the image
*/

/// \brief Perform a blob analysis on a grayscale image.
///
/// The grayscale source image is expected in a \a sourceGray, the labelled image will be
/// stored in \a destLabel. The feautures that should be calculated are passed via 
/// the BlobFeautureList object provided with \a blobFeatureList.
/// The resulting BlobResultList object contains the results of the blob analysis.
///
/// The image will be thresholded before analysis. The threshold used can be set
/// with mvIMPACT::blob::BlobFeatureList::setThreshold.
///
/// \see mvIMPACT::blob::BlobFeatureList
/// \see mvIMPACT::blob::BlobResultList
BlobResultList analyzeGray(
		       ///the grayscale source image
		       const Image& sourceGray, 
		       ///variable that receives the labeled image
		       Image& destLabel, 
		       ///list that contains the desired features
		       const BlobFeatureList& blobFeatureList);



/// \brief Perform a blob analysis on a grayscale image.
///
/// The grayscale source image is expected in a \a sourceDest,
///The feautures that should be calculated are passed via 
/// the BlobFeautureList object provided with \a blobFeatureList.
/// The resulting BlobResultList object contains the results of the blob analysis.
///
/// The image will be thresholded before analysis. The threshold used can be set
/// with mvIMPACT::blob::BlobFeatureList::setThreshold.
///
/// \see mvIMPACT::blob::BlobFeatureList
/// \see mvIMPACT::blob::BlobResultList
BlobResultList analyzeGray(
				/// the grayscale source image. After analyzation it contains the labeled image
				Image& sourceDest, 
				///list that contains the desired features
				const BlobFeatureList& blobFeatureList);


/// \brief Perform analysis of a single blob with the features provided with \a blobFeatureList.
///
/// A single blob can be acquired from mvIMPACT::BlobLocatorResultList after a call
/// of mvIMPACT::locateBlobs. 
///The returned BlobResultList object contains the results of the blob analysis.
/// \see mvIMPACT::blob::BlobFeatureList
//-----------------------------------------------------------------------------
BlobResultList analyzeSingleBlob(///the blob to be analyzed
				const SingleBlob& singleBlob,
		       ///the list of desired features
		       const BlobFeatureList& blobFeatureList);


/// \brief Transform a point \a point from the coordinate system given in \a transformation.
///
/// \a transformation defines a coordinate system relative to another one, called "base coordinate
/// system" in the following. This function transforms the point \a point specified
/// in the coordinate system given with \a point into the base coordinate system.
PointDouble transform(///the transformation
		      Position transformation, 
		      ///the point to transform
		      const PointDouble& point);



/// \brief Checks if the blob license is valid.
bool checkLicenseBlob();

/// \brief Return version number from blob module.

/// This function returns a version string. The following format is used: 
/// "major.minor.point.build" where "major" specifies the major version number, "minor" specifies
/// the minor version number, "point" specifies a point version number and "build" specifies the build number. 
/// Example: 6.3.1.4529
//-----------------------------------------------------------------------------
std::string getVersionBlob();


/// \brief Return version number from vl_blobloc module.

/// This function returns a version string. The following format is used: 
/// "major.minor.point.build" where "major" specifies the major version number, "minor" specifies
/// the minor version number, "point" specifies a point version number and "build" specifies the build number. 
/// Example: 6.3.1.4529
//-----------------------------------------------------------------------------
std::string getVersionBlobLocator();


	} // namespace blob
} // namespace mvIMPACT


#include "mvIMPACT_blob_impl.h"

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_PYTHON

#ifndef MVIMPACT_USE_NAMESPACES
using namespace mvIMPACT::blob;
#endif // #ifndef MVIMPACT_USE_NAMESPACES

#endif // #ifndef WRAP_PYTHON
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

#endif // MVIMPACT_BLOB_H_
