#ifndef MVIMPACT_OCR_H_
#define MVIMPACT_OCR_H_


/// \file
///

#ifndef WRAP_PYTHON
#include "mvIMPACT.h"
#endif // #ifndef WRAP_PYTHON


#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_PYTHON


#include <vl_ocr.h>


#endif // #ifndef WRAP_PYTHON
#endif // DOXYGEN_SHOULD_SKIP_THIS


namespace mvIMPACT {

        /// \namespace mvIMPACT::ocr This namespace contains all classes and functions from mvIMPACT OCR.
	namespace ocr {


/// \brief Ocr classification mode.

/// This enumeration lists the possible 
/// classification methods.
enum OcrClassification
{
	// Invalid classification method.
	//ocNone = mvIMPACT_C::OCR_NO_CLAS,
	
        /// Correlation method. Uses size normalization along with gray-level normalization
	ocCorrelation = mvIMPACT_C::OCR_KOR_CLAS,
	/// Fast Fourier method. Provides size invariance and rotation invariance, allowing to read text of an arbitrary orientation.
	ocFastFourier = mvIMPACT_C::OCR_DFT_CLAS,
};


/// \brief This class represents a single character in an OCR database. 

///A character 
/// consists of an image containing the character pattern and an ascii code
/// for that pattern.
//-----------------------------------------------------------------------------
class OcrCharacter
//-----------------------------------------------------------------------------
{
public:
	/// \brief construct a OcrCharacter object.
	///
	/// The default values are an unallocated image and \a ascii set to 0.
	OcrCharacter(
		/// the pattern
		const Image & pattern = Image(),
		///the corresponding ascii value
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="255"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int ascii = 0);
	
	/// Get pattern Image.	
	Image getPattern() const;

#ifndef WRAP_ANY
	/// Get pattern Image.	
	Image & getPattern();
#endif // #ifndef WRAP_ANY

	/// Set pattern Image.	
	OcrCharacter& setPattern(const Image & pattern);

	/// Get ascii character.
	int getAscii() const;
	
#ifndef WRAP_ANY
	/// Get ascii character.
	int & getAscii();
#endif // #ifndef WRAP_ANY

	/// Set ascii character.
	OcrCharacter& setAscii(
		///ASCII value
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="255"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int ascii);
		
private:
	/// Image of the character.
	Image m_pattern;
	/// Ascii code of the character.
	int   m_ascii;
};



/// \brief This class holds the result of a character recognition.

//-----------------------------------------------------------------------------
class OcrResult
//-----------------------------------------------------------------------------
{
public:
	
	/// \brief Create an empty OcrResult object
	///
	/// Note that no default initialization is done here.
	OcrResult();

	/// \brief Create an OcrResult object with given ASCII character, quality and angle in degrees.
	OcrResult(
		///ASCII value
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="255"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int ascii,
		/// the quality value
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="1.0"/></mvParamMax>
		/// <mvParamStep>0.1</mvParamStep>
		/// </mvParamRange>
		/// \endxmlonly			
		double quality, 
		/// rotation angle in degrees
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="360.0"/></mvParamMax>
		/// <mvParamStep>0.1</mvParamStep>
		/// </mvParamRange>
		/// \endxmlonly			
		double angle);

	/// \brief Create an OcrResult object with given ASCII character and quality.
	OcrResult(
		///ASCII value
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="255"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int ascii, 
		/// quality value
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="1.0"/></mvParamMax>
		/// <mvParamStep>0.1</mvParamStep>
		/// </mvParamRange>
		/// \endxmlonly			
		double quality);


	/// \brief Retrieve ASCII code of an analysed character.
	int getAscii() const;
	
#ifndef WRAP_ANY
	/// \brief Retrieve ASCII code of an analysed character.
	int & getAscii();
#endif // #ifndef WRAP_ANY

	/// \brief Set ASCII code of an analysed character.
	OcrResult& setAscii(
		/// ASCII value
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="255"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int ascii);
	
	/// \brief Retrieve quality of the recognition.
	///
	///This value is in the range [0,1]. The quality 
	/// is an indicator of the reliability of the recognition.
	double getQuality() const;
	
#ifndef WRAP_ANY
	/// \brief Retrieve quality of the recognition.
	///
	///This value is in the range [0,1]. The quality 
	/// is an indicator of the reliability of the recognition.
	double & getQuality();
#endif // #ifndef WRAP_ANY

	/// \brief Set quality of the recognition.
	///
	///This value is in the range [0,1]. The quality 
	/// is an indicator of the reliability of the recognition.
	OcrResult& setQuality(
		/// quality value
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="1.0"/></mvParamMax>
		/// <mvParamStep>0.1</mvParamStep>
		/// </mvParamRange>
		/// \endxmlonly			
		double quality);


	/// \brief Retrieve the angle of the character
	///
	/// This value is the angle of the character in degrees.
	/// The angle is only delivered in DFT mode in correlation mode
	/// this value is always 0.
	double getAngle()const;

#ifndef WRAP_ANY
	/// \brief Retrieve the angle of the character
	///
	/// This value is the angle of the character in degrees.
	/// The angle is only delivered in DFT mode in correlation mode
	/// this value is always 0.
	double & getAngle();
#endif // #ifndef WRAP_ANY

	/// \brief Set the angle of the character
	///
	/// This value is the angle of the character in degrees.
	/// The angle is only delivered in DFT mode in correlation mode
	/// this value is always 0.
        OcrResult& setAngle(
			///angle in degrees
			/// \xmlonly
			/// <mvParamRange>
			/// <mvParamMin><mvParamConst value="0.0"/></mvParamMin>
			/// <mvParamMax><mvParamConst value="360.0"/></mvParamMax>
			/// <mvParamStep>0.1</mvParamStep>
			/// </mvParamRange>
			/// \endxmlonly			
			double angle);


#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_PYTHON
	operator mvIMPACT_C::OCR_RESULT() const;
	OcrResult(const mvIMPACT_C::OCR_RESULT& r);
#endif // #ifndef WRAP_PYTHON
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

private:

	int   m_ascii;
	double m_quality;
	double m_angle;
};


class OcrFontDatabase;

/// \brief The SegmentList contains a list of rectangles where characters are
/// assumed to be. 

/// This list is filled by mvIMPACT::ocr::OcrFontDatabase::segment.
/// The list of rectangles is always filled from left to right. That means
/// from lower to higher indexes. There must be no gaps in the list.
//-----------------------------------------------------------------------------
class SegmentList
//-----------------------------------------------------------------------------
{
private:
	/// A rectangle array.
	mvIMPACT_C::IPL_RECT*   m_pData;
	/// The size of the array.
	int m_iCapacity;
	/// The number of elements in the array.
	int m_iSize;

public:
	///\brief Construct a SegmentList which can store a maximum number of \a capacity 
	/// elements. 
	///
    ///Each element is a Rectangle.
	SegmentList (
		/// capacity
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="255"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int capacity = 100);

	/// \brief Free memory allocated by SegmentList.
	~SegmentList ();

	///\brief Construct a SegmentList from \a src
	SegmentList (const SegmentList& src);
#ifndef WRAP_PYTHON
	/// \brief Overwrite current object with \a src.
	SegmentList& operator= (const SegmentList& src);
#endif // #ifndef WRAP_PYTHON


	/// \brief Sort all segments. 

	///Sorting is done from left to right and then from top to down.
	/// The result is the natural reading order.
	void sort
	(
		/// The OCR font database to use.
		const OcrFontDatabase& ocrFontDatabase,
		/// The image that segmented in order to create this SegmentList.
		const Image& source
	);

	/// \brief Split up segments which possibly contain several characters

	///Analyzes the segment list on the basis of the permissible character size 
	/// and tries to split up segments which possibly contain several characters.
	///This only makes sense in correlation mode, since in DFT mode the teaching
	/// and reading/verifying functions do not support splitted segments.
	/// \see mvIMPACT::ocr::OcrFontDatabase::setMinCharWidth
	///\see mvIMPACT::ocr::OcrFontDatabase::setMinCharHeight
	/// \see mvIMPACT::ocr::OcrFontDatabase::setMaxCharWidth
	///\see mvIMPACT::ocr::OcrFontDatabase::setMaxCharHeight
	///\see mvIMPACT::ocr::SegmentList::merge
	void split
	(
		/// The OCR font database to use.
		const OcrFontDatabase&  ocrFontDatabase,
		/// The image that was segmented in order to create this SegmentList.
		const Image& source
	);

	///\brief Merge small segments, that possibly make up only one character

	/// Analyzes the segment list on the basis of the permissible character size  
	/// and tries to combine multiple adjacent small segments.
	///This only makes sense in correlation mode, since in DFT mode the teaching
	/// and reading/verifying functions do not support merged segments.
	/// \see mvIMPACT::ocr::OcrFontDatabase::setMinCharWidth
	/// \see mvIMPACT::ocr::OcrFontDatabase::setMinCharHeight
	/// \see mvIMPACT::ocr::OcrFontDatabase::setMaxCharWidth
	///\see mvIMPACT::ocr::OcrFontDatabase::setMaxCharHeight
	///\see mvIMPACT::ocr::SegmentList::split
	void merge
	(
		/// The OCR font database to use.
		const OcrFontDatabase& ocrFontDatabase,
		/// The image that was segmented in order to create this SegmentList.
		const Image& source
	);


	/// \brief Return the number of rectangles in the list. 
	///
	///The last rectangle in the list has
	/// the index getSize()-1.
	///\see mvIMPACT::ocr::SegmentList::setSize
	int getSize() const;
	
	/// \brief Set the number of rectangles in the list. 
	///
	///This value must not be greater than
	///the capacity of the list. It is truncated to capacity if it is larger.
	///\see mvIMPACT::ocr::SegmentList::getSize
	///\see mvIMPACT::ocr::SegmentList::setCapacity
	SegmentList& setSize (
			///the actual size of the SegmentList
			/// \xmlonly
			/// <mvParamRange>
			/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
			/// <mvParamMax><mvParamComplex Python="%(instance)s.capacity-1"/></mvParamMax>
			/// </mvParamRange>
			/// \endxmlonly			
			int size);
	

	/// \brief Return the maximum number of rectangles that this list can hold.
	///
	///\note This is not the actual size of the list.
	///\see mvIMPACT::ocr::SegmentList::setCapacity
	///\see mvIMPACT::ocr::SegmentList::getSize
	int getCapacity() const;
	
	/// \brief Set the maximum number of rectangles that this list can hold.
	///
	/// The initial capacity of the SegmentList can be set in the constructor.
	///\note This is not the actual size of the SegmentList.
	///\see mvIMPACT::ocr::SegmentList::getCapacity
	///\see mvIMPACT::ocr::SegmentList::setSize
	///\see mvIMPACT::ocr::SegmentList::SegmentList
	SegmentList& setCapacity (
			///the maximum number of rectangles this SegmentList can hold
			/// \xmlonly
			/// <mvParamRange>
			/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
			/// <mvParamMax><mvParamConst value="255"/></mvParamMax>
			/// </mvParamRange>
			/// \endxmlonly			
			int capacity);
	

	/// \brief Access a particular segment.
	///	
	/// Using this method you can gather information about the segment, like its position and dimensions.
	/// \see mvIMPACT::Rectangle
	/// \see mvIMPACT::ocr::setSegment
	Rectangle getSegment 
	(
		/// Index of the segment. It must be in the range [0,mvIMPACT::ocr::SegmentList::getSize -1]. 
		/// Passing other values results in an exception.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.size-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int index
		) const;
	
	/// \brief Change a segment in the list. 
	///
	/// This function doesn't call mvIMPACT::ocr::SegmentList::setSize to adjust
	/// the number of elements in the list. It is the task of the user to assure that
	/// there are no gaps in the list and that the size is correct
	/// \see mvIMPACT::Rectangle
	/// \see mvIMPACT::ocr::getSegment
	SegmentList& setSegment 
	(
		/// Index of the segment. It must be in the range [0,mvIMPACT::ocr::SegmentList::getCapacity -1]. 
		/// Passing other values results in an exception.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.capacity-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int index, 
		/// The segment as a mvIMPACT::Rectangle.
		const Rectangle& rect
		);
	

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_PYTHON
	/// Return a pointer to the interal Rectangle array. With this pointer you can access
	/// the rectangle data directly.
	mvIMPACT_C::IPL_RECT* getData() const;
#endif // #ifndef WRAP_PYTHON
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

private:
	void dispose();
	
	void copy(const SegmentList& src);
};


/// \brief This class represents an OCR font database. 

///It is both used for teaching 
/// and for recognizing characters. A font database contains any number of 
/// characters. Each character is reprensented by an image and the ascii code 
/// that it stands for. The most common usage is to load an existing OCR font 
/// database from a file. These files have the extension '.ocr'.
/// \note Only one instance of OcrFontDatabase can exist at the same time!
/// 
//-----------------------------------------------------------------------------
class OcrFontDatabase
//-----------------------------------------------------------------------------
{
public:
	/// \brief Construct an OCR database. 
	///
    ///If \a mustExist is \c true then the passed file 
	/// name must point to an existing database. If the flag is \c false a new database
	/// is created if it does not exist already.
	OcrFontDatabase (
		/// Name of *.OCR file with pattern.
		const std::string& fileName,
		/// Creation flags: \c true if the *.OCR file must exist, \c false to create a new database
		/// file if it does not exist already.
		bool mustExist = true);
	

	/// \brief Construct an empty unnamed OCR database. 

	///Always use the other constructor.
	/// The only purpose of this constructor is to allow a default construction.
	OcrFontDatabase();

	/// \brief Destruct a font database.
	virtual ~OcrFontDatabase();

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_ANY
	mvIMPACT_C::OCR_BUFHANDLE getHandle  () const; 
	int getUseCount() const; 
	bool getIsUnique   () const; 
#endif // #ifndef WRAP_ANY
#endif
	
#ifndef WRAP_PYTHON
	/// \brief Overwrite current object with \a src.
	OcrFontDatabase& operator=(const OcrFontDatabase& src);
#endif // #ifndef WRAP_PYTHON

	/// \brief Construct OcrFontDatabase object from \a src.
	OcrFontDatabase(const OcrFontDatabase& src);

public:
	/// \brief Teach the OCR database a new character.

	/// Extracts a rectangular area from image \a source and asociates the ascii character \a ascii with it.
	/// In correlation mode the whole 
	/// pattern contained in the rectangle will be stored in the OCR database. 
	/// In DFT mode a new segmentation
	/// is performed, thus the segment stored in the OCR database 
	/// as a result of mvIMPACT::ocr::OcrFontDatabase::addCharacter can be smaller than the rectangle given with 
	/// parameter \c segmentRect. In the latter case you have to preprocess the image 
	/// in a way that only one segment is found. Splitting and merging segments do
	/// not apply for teaching in DFT mode!
	/// \note Take care that you choose the correct classification mode during training:
	/// \note if you choose Discrete Fourier Transform your database will work in fourier mode AND in correlation mode
	/// \note if you choose Correlation your database will work in correlation mode ONLY

	/// \see mvIMPACT::ocr::OcrFontDatabase::delCharacter
	void addCharacter 
	(
		/// Source image where the pattern for the new character is extracted.
		const Image& source, 
		/// A rectangular area in the source image where the new character is
		/// located.
		const Rectangle& segmentRect,
		/// The ascii code that is associated with this pattern.
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="255"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int ascii
		);
	

	/// Remove a character from the OCR database.

	///Deletes the character with index \a index from the database.
	/// \see mvIMPACT::ocr::OcrFontDatabase::addCharacter
	/// \see mvIMPACT::ocr::OcrFontDatabase::getCharacterCount
	void delCharacter 
	(
		/// Index of the character. Valid in the range [0,mvIMPACT::ocr::OcrFontDatabase::getCharacterCount -1].
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.characterCount-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int index
		);
	

	/// \brief Retrieve information about a particular character in the OCR database.
	
	/// Use this function if you want a mvIMPACT::Image of the character and the ascoiated ASCII code.
	/// \see mvIMPACT::ocr::OcrFontDatabase::getCharacterCount
	/// \see mvIMPACT::ocr::OcrCharacter
	OcrCharacter getCharacterInfo 
	(
		/// Index of the character. Valid in the range [0,mvIMPACT::ocr::OcrFontDatabase::getCharacterCount -1].
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.characterCount-1"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int index
		) const;
	

public:
	/// \brief Segment an image into candidate areas where characters are assumed. 

	///The result of this
	/// step depends on the segmentation settings. Look at the member functions of this class.
	/// \note Ensure that your SegmentLists capacity is large enough, to avoid that some segments are discarded.
	/// \see mvIMPACT::ocr::OcrFontDatabase::setMinSegWidth
	/// \see mvIMPACT::ocr::OcrFontDatabase::setMinSegHeight
	/// \see mvIMPACT::ocr::OcrFontDatabase::setMaxSegWidth
	/// \see mvIMPACT::ocr::OcrFontDatabase::setMaxSegHeight
	/// \see mvIMPACT::ocr::OcrFontDatabase::setBlackOnWhite
	void segment
	(
		/// Source image. Instead of a whole Image you can also pass a part that contains
		/// an interesting area to speed up the OCR.
		const Image& source,
		/// This list must be large enough to hold all results. If there are more results
		/// than place in this list the excessive segments are discarded.
		SegmentList& segmentList
		);
	

	/// \brief Read a character in a specified mvIMPACT::Rectangle.

	///The character from the OCR database that matches best is returned in
	/// an mvIMPACT::ocr::OcrResult object. To read all characters of an image use mvIMPACT::ocr::OcrFontDatabase::readString.
	/// \see mvIMPACT::Rectangle
	/// \see mvIMPACT::ocr::OcrResult
	/// \see mvIMPACT::ocr::SegmentList
	OcrResult readChar
	(
		/// The source image.
		const Image& source,
		/// A rectangle that contains the image part where the character is located.
		const Rectangle& segmentRect
		) const;
	
	

	/// \brief Read a character and return the first \a numberOfResults characters from the OCR database 
	/// that match best.
	///
	/// The std::vector object contains mvIMPACT::ocr::OcrResult objects that  are sorted
	/// by decreasing matching score.
	/// To read all characters of an image iterate through
	/// all segments in your mvIMPACT::ocr::SegmentList, get the asociated mvIMPACT::Rectangle objects
	/// and pass them as \a segmentRect. Then you can query the ASCII code with mvIMPACT::ocr::OcrResult::ascii
	/// \note The number of items in the result is not necessarly equal to the desired number of results 
	/// \note specified with \a numberOfResults .
	/// \see mvIMPACT::Rectangle
	/// \see mvIMPACT::ocr::OcrResult
	/// \see mvIMPACT::ocr::SegmentList
	std::vector<OcrResult> readChar
	(
		/// The source image.
		const Image& source,
		/// A rectangle that contains the image part where the character is located.
		const Rectangle& segmentRect,
		/// Specify how many results you want to fetch. 
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			
		int numberOfResults
		) const;
	

	/// \brief Read a string from a specified segment list.
	///
	/// For each segment, the character achieving the best score is returned. The scores of the characters
	/// are discarded here. Use mvIMPACT::ocr::OcrFontDatabase::readChar to get them for
	/// a single segment.
	/// 
	/// Newlines and spaces are inserted to separate words and lines as follows:
	/// If the distance between two segments is greater than the mean width of all segments,
	/// a space is inserted between the two segments. If the x coordinate of a segment is less than the x coordinate of
	/// the previous segment, a newline is inserted between the two segments.
	/// This rules work best, if SegmentList::sort has been called before.
	std::string readString
	(
		/// The source image.
		const Image& source,
		/// The list of segments. Returned by a call to mvIMPACT::ocr::OcrFontDatabase::segment, for example.
		const SegmentList& segmentList
	) const;			



	/// \brief Compare a character in a specified mvIMPACT::Rectangle with a specified pattern in the OCR database.

	/// The result of the comparisson between the character in the given rectangle and the character at the specified 
	/// index in the OCR database is returned in an mvIMPACT::ocr::OcrResult object.
	/// \see mvIMPACT::Rectangle
	/// \see mvIMPACT::ocr::OcrResult
	/// \see mvIMPACT::ocr::SegmentList
	OcrResult compareChar
	(
		/// The source image.
		const Image& source,
		/// A rectangle that contains the image part where the character is located.
		const Rectangle& segmentRect,
		/// the index of the character in the OCR database
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamComplex Python="%(instance)s.characterCount-1"/></mvParamMax>		
		/// </mvParamRange>
		/// \endxmlonly			
		int index
		) const;




	/// \brief Compute matching scores for a given range of characters. 

	///You can use this function
	/// for the following case: You know that a particular image segment contains the characters
	/// 'A' or 'B'. What is the matching score assumed it is 'A or 'B'?
	/// The result of this function is a vector of floating points values that contain the
	/// matching scores of the passed in characters.
	/// Example:
	/// \code
	/// OcrFontDatabase db("mydb.ocr"); 
	/// Image img(loadImage("image.bmp"));
	/// Rectangle rect(10,10,30,60);
	/// vector<int> vChars;
	/// vChars.push_back('A');
	/// vChars.push_back('B');
	/// vector<float> vScores = db.verifyChar(img,rect,vChars);
	/// double scoreForA = vScores[0];
	/// double scoreForB = vScores[1];
	/// \endcode
	std::vector<double> verifyChar
	(
		/// The source image.
		const Image&        S,
		/// A rectangle that contains the image part where the character is located.
		const Rectangle& segmentRect,
		/// A vector of integers. Each entry is the ascii code of one of the possible
		/// characters.
		const std::vector<int>& PYTHON_CPP_SWITCH(SEQUENCE_TO_VECTOR,asciis)
		) const;
	

public: //properties


	/// \brief Retrieve the current threshold used to binarize the source image.
	///
	/// \see mvIMPACT::ocr::OcrFontDatabase::setThreshold
	int getThreshold() const; 
	


	/// \brief Set the threshold used to binarize the source image.
	///
	/// \see mvIMPACT::ocr::OcrFontDatabase::setThreshold
	OcrFontDatabase& setThreshold(
		///the new threshold value
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// <mvParamMax><mvParamConst value="255"/></mvParamMax>
		/// </mvParamRange>
		/// \endxmlonly			
		int threshold); 
		


	/// \brief Return the operation mode of mvIMPACT OCR
	///
	/// Returns \c true for operation mode 'black characters on white background'.
	/// Returns \c false for operation mode 'white characters on black background'.
	/// \see mvIMPACT::ocr::OcrFontDatabase::setBlackOnWhite
	bool getBlackOnWhite() const; 
	



	///\brief Set the mvIMPACT OCR operation mode.
	///
	/// Specify whether the characters to read are black on a white background or vice versa.
	/// Set \c true for operation mode 'black characters on white background'.
	/// Set \c false for operation mode 'white characters on black background'.
	/// \see mvIMPACT::ocr::OcrFontDatabase::getBlackOnWhite
	OcrFontDatabase& setBlackOnWhite
	(
		/// \c true means black characters on a white background. 
		/// \c false means white characters on a black background.
		bool blackOnWhite
		);
		
	/// \brief Retrieve the minimum allowed segment width. 

	/// This parameter is used during image 
	/// segmentation to decide which segments are valid. 
	/// \see mvIMPACT::ocr::OcrFontDatabase::segment
	/// \see mvIMPACT::ocr::OcrFontDatabase::setMinSegWidth
	/// \see mvIMPACT::ocr::OcrFontDatabase::getMaxSegWidth
	int getMinSegWidth() const; 
	
	/// \brief Set the minimum allowed segment width. 

	///This parameter is used during image 
	/// segmentation to decide which segments are valid. 
	/// \see mvIMPACT::ocr::OcrFontDatabase::segment
	/// \see mvIMPACT::ocr::OcrFontDatabase::getMinSegWidth
	/// \see mvIMPACT::ocr::OcrFontDatabase::setMaxSegWidth
	OcrFontDatabase& setMinSegWidth(
		///the minimal segment width
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			
		int minSegWidth); 
		
	/// \brief Retrieve the maximum allowed segment width. 

	/// This parameter is used during image 
	/// segmentation to decide which segments are valid. 
	/// \see mvIMPACT::ocr::OcrFontDatabase::segment
	/// \see mvIMPACT::ocr::OcrFontDatabase::setMaxSegWidth
	/// \see mvIMPACT::ocr::OcrFontDatabase::getMinSegWidth
	int getMaxSegWidth() const; 
	
	/// \brief Set the maximum allowed segment width. 
	
	///This parameter is used during image 
	/// segmentation to decide which segments are valid. 
	/// \see mvIMPACT::ocr::OcrFontDatabase::segment
	/// \see mvIMPACT::ocr::OcrFontDatabase::getMaxSegWidth
	/// \see mvIMPACT::ocr::OcrFontDatabase::setMinSegWidth
	OcrFontDatabase& setMaxSegWidth(
		///the maximal segment width
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			
		int maxSegWidth); 
		

	/// \brief Retrieve the minimum allowed segment height. 
	///
	/// This parameter is used during image 
	/// segmentation to decide which segments are valid. 
	/// \see mvIMPACT::ocr::OcrFontDatabase::segment
	/// \see mvIMPACT::ocr::OcrFontDatabase::setMinSegHeight
	/// \see mvIMPACT::ocr::OcrFontDatabase::getMaxSegHeight
	int getMinSegHeight() const; 
	
	/// \brief Set the minimum allowed segment height. 

	///This parameter is used during image 
	/// segmentation to decide which segments are valid. 
	/// \see mvIMPACT::ocr::OcrFontDatabase::segment
	/// \see mvIMPACT::ocr::OcrFontDatabase::getMinSegHeight
	/// \see mvIMPACT::ocr::OcrFontDatabase::setMaxSegHeight
	OcrFontDatabase& setMinSegHeight(
		///the minimal segment height
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			
		int minSegHeight); 
		

	/// \brief Retrieve the maximum allowed segment height. 

	///This parameter is used during image 
	/// segmentation to decide which segments are valid. 
	/// \see mvIMPACT::ocr::OcrFontDatabase::segment
	/// \see mvIMPACT::ocr::OcrFontDatabase::setMaxSegHeight
	/// \see mvIMPACT::ocr::OcrFontDatabase::getMinSegHeight
	int getMaxSegHeight() const; 
	
	/// \brief Set the maximum allowed segment height. 

	///This parameter is used during image 
	/// segmentation to decide which segments are valid. 
	/// \see mvIMPACT::ocr::OcrFontDatabase::segment
	/// \see mvIMPACT::ocr::OcrFontDatabase::getMaxSegHeight
	/// \see mvIMPACT::ocr::OcrFontDatabase::setMinSegHeight
	OcrFontDatabase& setMaxSegHeight(
		///the maximal segment height
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			
		int maxSegHeight); 
	
	/// \brief Retrieve the minimum allowed character width. 

	///This setting is used by 
	/// mvIMPACT::ocr::SegmentList::split and mvIMPACT::ocr::SegmentList::merge to correct the segments.
	///\see mvIMPACT::ocr::OcrFontDatabase::setMinCharWidth
	///\see mvIMPACT::ocr::OcrFontDatabase::getMaxCharWidth
	///\see mvIMPACT::ocr::SegmentList::split
	///\see mvIMPACT::ocr::SegmentList::merge
	int getMinCharWidth() const; 
	
	/// \brief Set the minimum allowed character width. 

	///This setting is used by 
	/// mvIMPACT::ocr::SegmentList::split and mvIMPACT::ocr::SegmentList::merge to correct the segments.
	///This setting is used by 
	/// mvIMPACT::ocr::SegmentList::split and mvIMPACT::ocr::SegmentList::merge to correct the segments.
	///\see mvIMPACT::ocr::OcrFontDatabase::getMinCharWidth
	///\see mvIMPACT::ocr::OcrFontDatabase::setMaxCharWidth
	///\see mvIMPACT::ocr::SegmentList::split
	///\see mvIMPACT::ocr::SegmentList::merge
	OcrFontDatabase& setMinCharWidth(
		///the minimal character width
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			
		int minCharWidth); 
		
	/// \brief Retrieve the maximum allowed character width. 

	///This setting is used by 
	/// mvIMPACT::ocr::SegmentList::split and mvIMPACT::ocr::SegmentList::merge to correct the segments.
	///\see mvIMPACT::ocr::OcrFontDatabase::setMaxCharWidth
	///\see mvIMPACT::ocr::OcrFontDatabase::getMinCharWidth
	///\see mvIMPACT::ocr::SegmentList::split
	///\see mvIMPACT::ocr::SegmentList::merge
	int getMaxCharWidth() const; 
	
	/// \brief Set the maximum allowed character width.

	///This setting is used by 
	/// mvIMPACT::ocr::SegmentList::split and mvIMPACT::ocr::SegmentList::merge to correct the segments.
	///\see mvIMPACT::ocr::OcrFontDatabase::getMaxCharWidth
	///\see mvIMPACT::ocr::OcrFontDatabase::setMinCharWidth
	///\see mvIMPACT::ocr::SegmentList::split
	///\see mvIMPACT::ocr::SegmentList::merge
	OcrFontDatabase& setMaxCharWidth(
		///the maximal character width
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			
		int maxCharWidth); 
		
	/// \brief Retrieve the minimum allowed character height. 

	///This setting is used by 
	/// mvIMPACT::ocr::SegmentList::split and mvIMPACT::ocr::SegmentList::merge to correct the segments.
	///\see mvIMPACT::ocr::OcrFontDatabase::setMinCharHeight
	///\see mvIMPACT::ocr::OcrFontDatabase::getMaxCharHeight
	///\see mvIMPACT::ocr::SegmentList::split
	///\see mvIMPACT::ocr::SegmentList::merge
	int getMinCharHeight() const; 
	
	/// \brief Set the minimum allowed character height. 

	///This setting is used by 
	/// mvIMPACT::ocr::SegmentList::split and mvIMPACT::ocr::SegmentList::merge to correct the segments.
	///\see mvIMPACT::ocr::OcrFontDatabase::getMinCharHeight
	///\see mvIMPACT::ocr::OcrFontDatabase::setMaxCharHeight
	///\see mvIMPACT::ocr::SegmentList::split
	///\see mvIMPACT::ocr::SegmentList::merge
	OcrFontDatabase& setMinCharHeight(
		///the minimal character height
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			
		int minCharHeight); 
		
	/// \brief Retrieve the maximum allowed character height. 

	///This setting is used by 
	/// mvIMPACT::ocr::SegmentList::split and mvIMPACT::ocr::SegmentList::merge to correct the segments.
	///\see mvIMPACT::ocr::OcrFontDatabase::setMaxCharHeight
	///\see mvIMPACT::ocr::OcrFontDatabase::getMinCharHeight
	///\see mvIMPACT::ocr::SegmentList::split
	///\see mvIMPACT::ocr::SegmentList::merge
	int getMaxCharHeight() const; 
	
	/// \brief Set the maximum allowed character height. 

	///This setting is used by 
	/// mvIMPACT::ocr::SegmentList::split and mvIMPACT::ocr::SegmentList::merge to correct the segments.
	///\see mvIMPACT::ocr::OcrFontDatabase::getMaxCharHeight
	///\see mvIMPACT::ocr::OcrFontDatabase::setMinCharHeight
	///\see mvIMPACT::ocr::SegmentList::split
	///\see mvIMPACT::ocr::SegmentList::merge
	OcrFontDatabase& setMaxCharHeight(
		///the maximal character height
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="0"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			
		int maxCharHeight); 
		
	/// \brief Retrieve flag that indicates, whether the mvIMPACT::ocr::SegmentList that is returned by 
	/// \brief mvIMPACT::ocr::OcrFontDatabase::segment is sorted automatically or not.
	
	///Returns \c true if it is sorted automatically
	/// Returns \c false if it is not sorted automatically
	/// \see mvIMPACT::ocr::OcrFontDatabase::segment
	/// \see mvIMPACT::ocr::OcrFontDatabase::setSortFlag
	/// \see mvIMPACT::ocr::SegmentList
	bool getSortFlag() const; 
	
	/// \brief Specify whether the mvIMPACT::ocr::SegmentList that is returned by 
	/// mvIMPACT::ocr::OcrFontDatabase::segment is sorted automatically or not.

	/// The default is not to sort the list.
	/// Set to \c true if you want to sort it automatically
	/// Set to \c false if you don't want to sort it automatically.
	/// \see mvIMPACT::ocr::OcrFontDatabase::segment
	/// \see mvIMPACT::ocr::OcrFontDatabase::getSortFlag
	/// \see mvIMPACT::ocr::SegmentList
	OcrFontDatabase& setSortFlag(///the new value of the sort flag
			 bool sortFlag); 
	
	/// \brief Return the number of characters in the OCR database.
	
	///
	/// \see mvIMPACT::ocr::OcrFontDatabase::addCharacter
	/// \see mvIMPACT::ocr::OcrFontDatabase::delCharacter
	int getCharacterCount() const; 
	


   	/// \brief Retrieve the method used to classify characters.

	///The default is to use a correlation
	/// based classification method.
	/// \see mvIMPACT::ocr::OcrFontDatabase::setClassificationType
	/// \see mvIMPACT::ocr::OcrClassification
	OcrClassification getClassificationType() const; 
	
	/// \brief Set the method used to classify characters. 

	///The default is to use a correlation
	/// based classification method.
	/// \see mvIMPACT::ocr::OcrFontDatabase::getClassificationType
	/// \see mvIMPACT::ocr::OcrClassification
	OcrFontDatabase& setClassificationType(///the new classification method
				   OcrClassification classificationType); 
		

	/// \brief Return the current normalization setting.

	///If \c true is returned all characters in the database are normalized to the same width and
	/// height. Also the segments passed in with mvIMPACT::ocr::OcrFontDatabase::readChar and mvIMPACT::ocr::OcrFontDatabase::verifyChar are normalized.
	/// By default this normalization is done.
	/// \see mvIMPACT::ocr::OcrFontDatabase::setNormalizeFlag
	/// \see mvIMPACT::ocr::OcrFontDatabase::readChar 
	/// \see mvIMPACT::ocr::OcrFontDatabase::verifyChar
	/// \see mvIMPACT::ocr::OcrFontDatabase::getNormalizeWidth 
	/// \see mvIMPACT::ocr::OcrFontDatabase::getNormalizeHeight
	bool getNormalizeFlag() const; 
	
	/// \brief Set the normalization flag.

	/// If set to \c true all characters in the database are normalized to the same width and
	/// height. Also the segments passed in with mvIMPACT::ocr::OcrFontDatabase::readChar and mvIMPACT::ocr::OcrFontDatabase::verifyChar are normalized.
	/// By default this normalization is done. Only switch this off if you know what you are 
	/// doing!
	/// \see mvIMPACT::ocr::OcrFontDatabase::setNormalizeFlag
	/// \see mvIMPACT::ocr::OcrFontDatabase::readChar 
	/// \see mvIMPACT::ocr::OcrFontDatabase::verifyChar
	/// \see mvIMPACT::ocr::OcrFontDatabase::setNormalizeWidth 
	/// \see mvIMPACT::ocr::OcrFontDatabase::setNormalizeHeight
	OcrFontDatabase& setNormalizeFlag(bool normalizeFlag); 

		
   	/// \brief Return the character width that is used when mvIMPACT::ocr::OcrFontDatabase::setNormalizeFlag is set to \c true.
	
	///If normalization is switched on, as it is by default, all characters in the OCR database
	/// are normalized to this width.
	/// \see mvIMPACT::ocr::OcrFontDatabase::getNormalizeFlag
	/// \see mvIMPACT::ocr::OcrFontDatabase::setNormalizeWidth 
	/// \see mvIMPACT::ocr::OcrFontDatabase::getNormalizeHeight
	int getNormalizeWidth() const; 
	
	/// \brief Set the character width that is used when mvIMPACT::ocr::OcrFontDatabase::setNormalizeFlag is set to \c true.

	/// If normalization is switched on, as it is by default, all characters in the OCR database
	/// are normalized to this width.
	/// \see mvIMPACT::ocr::OcrFontDatabase::setNormalizeFlag
	/// \see mvIMPACT::ocr::OcrFontDatabase::getNormalizeWidth 
	/// \see mvIMPACT::ocr::OcrFontDatabase::setNormalizeHeight
	OcrFontDatabase& setNormalizeWidth(
		///the character width
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			
		int normalizeWidth); 
		
	/// \brief Return the character height that is used when mvIMPACT::ocr::OcrFontDatabase::setNormalizeFlag is set to \c true.

	/// If normalization is switched on, as it is by default, all characters in the OCR database
	/// are normalized to this height.
	/// \see mvIMPACT::ocr::OcrFontDatabase::setNormalizeFlag
	/// \see mvIMPACT::ocr::OcrFontDatabase::setNormalizeHeight
	/// \see mvIMPACT::ocr::OcrFontDatabase::getNormalizeWidth 
	int getNormalizeHeight() const; 
	
	/// \brief Set the character height that is used when mvIMPACT::ocr::OcrFontDatabase::setNormalizeFlag is set to \c true.

	/// If normalization is switched on, as it is by default, all characters in the OCR database
	/// are normalized to this height.
	/// \see mvIMPACT::ocr::OcrFontDatabase::setNormalizeFlag
	/// \see mvIMPACT::ocr::OcrFontDatabase::getNormalizeHeight
	/// \see mvIMPACT::ocr::OcrFontDatabase::setNormalizeWidth 
	OcrFontDatabase& setNormalizeHeight(
		///the character height
		/// \xmlonly
		/// <mvParamRange>
		/// <mvParamMin><mvParamConst value="1"/></mvParamMin>
		/// </mvParamRange>
		/// \endxmlonly			
		int normalizeHeight); 
	


private:
	/// Unassigned buffer constant.
	enum { unassigned = -1 };
	/// The buffer handle.
	mvIMPACT_C::OCR_BUFHANDLE handle_;
	/// Pointer to the reference counter.
	mutable int* counter_;


	/// Dispose the buffer. If the reference count drops to zero,
	/// and the buffer was allocated it is freed.
	void dispose();
	
	void copy(const OcrFontDatabase& src);

	/// \brief Construct an allocated OcrFontDatabase object from a handle
	explicit OcrFontDatabase(
		/// the buffer handle
		mvIMPACT_C::OCR_BUFHANDLE  bufhandle);
};

/// \brief Return version number from vl_ocr module.

/// \brief Checks if the ocr license is valid.
bool checkLicenseOCR();

/// This function returns a version string. The following format is used: 
/// "major.minor.point.build" where "major" specifies the major version number, "minor" specifies
/// the minor version number, "point" specifies a point version number and "build" specifies the build number. 
/// Example: 6.3.1.4529
//-----------------------------------------------------------------------------
std::string getVersionOCR();

	} // namespace ocr
} // namespace mvIMPACT



#include "mvIMPACT_ocr_impl.h"

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#ifndef WRAP_PYTHON

#ifndef MVIMPACT_USE_NAMESPACES
using namespace mvIMPACT::ocr;
#endif // #ifndef MVIMPACT_USE_NAMESPACES

#endif // #ifndef WRAP_PYTHON
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

#endif // MVIMPACT_OCR_H_
