//----------------------------------------------------------------------
//----------------------------------------------------------------------
// MATRIX VISION GmbH
// mvIMPACT - mvBlueLYNX demo >>calib_live<<
// Fre Okt  8 13:39:43 CEST 2004
//----------------------------------------------------------------------
//----------------------------------------------------------------------
/*! \file calib_live.cpp
 *  \brief This is a sample for the \b mvIMPACT \b Calibration module. It uses live grabbed images.
 *
 * This sample is using grabbed (live) images to show how the mvIMPACT Calibration module
 * could be used. It is a \b console program.
 * After starting the sample a small usage dialog will be displayed on the console.
 * \par Usage Dialog:
 * \c L: by pressing \c L the sample will grab live images and will display them.
 * \c T: by pressing \c T a calibration \a teach will be made. The grabbed image will
 *       be used for this. Affter the calibration teach is finished the resulting statistics
 *       will be showen. If the resulting average values are grater \c 1.0 the computed
 *       calibration data might be not accurate enough. In this case the teach process
 *       could be restarted. Press \c Y to restart the teach process for the question
 *       \c Teach \c again? If the computed calibration data are OK (avarage data \c < \c 1.0)
 *       you can continue by pressing \c N. In this case the computed data will be saved
 *       into a file. If you don't want to save the new data into the file press \c Q.
 * \c C: by pressing \c C the calibration data from the teach process or if existing
 *       and loaded the calibration data from the file will be used to transform/calibrate
 *       the grabbed image.
 * \c Q: by pressing \c Q the sample will be terminated.
 *
 */
//----------------------------------------------------------------------
#include <stdio.h>

#include <vl_base.h>
#include <vl_device.h>
#include <vl_display.h>
#include <vl_camcal.h>
#include <vl_ipl.h>

#include <samples.h>

//----------------------------------------------------------------------
//----------------------------------------------------------------------
#define MYCHECKSDKCALL( _c )													\
		{																		\
			if( !( _c ))														\
			{																	\
				long _err = IPL_GetErrorCode2( );								\
				fprintf( stderr, "*** %s (%s: %d) - >>%s<< -> %s (%ld)!!\n",	\
									__FUNCTION__, __FILE__, __LINE__, #_c,		\
									IPL_GetErrorString( _err ), _err );			\
				DEV_Close( deviceHandle );										\
				abort( );														\
			}																	\
		}
//----------------------------------------------------------------------
//----------------------------------------------------------------------
#define TOOL_PREFIX				"calib_live"

//----------------------------------------------------------------------
//----------------------------------------------------------------------
static IPL_HWND hwnd = 0;
static IPL_BUFHANDLE imageHandle   = IPL_DONT_CARE;
static IPL_BUFHANDLE ccImageHandle = IPL_DONT_CARE;
static DEV_HANDLE deviceHandle     = IPL_DONT_CARE;
static CC_BUFHANDLE camcalHandle   = IPL_DONT_CARE;
static char *pCalFlName            = TOOL_PREFIX".ccl";

//----------------------------------------------------------------------
//----------------------------------------------------------------------
/* Free all used memory. */
static void FreeAllHandles( )
{
	if( deviceHandle != IPL_DONT_CARE )
    	DEV_Close( deviceHandle );

	if( camcalHandle != IPL_DONT_CARE )
		CC_BufFree( camcalHandle );

	if( ccImageHandle != IPL_DONT_CARE )
		IPL_BufFree( ccImageHandle );

	if( imageHandle != IPL_DONT_CARE )
		IPL_BufFree( imageHandle );
}

//----------------------------------------------------------------------
/* Display the menu on stdout. */
static void schowMenu( )
{
	printf( " == ---------------------------------------------\n" );
	printf( " == " TOOL_PREFIX ": "__DATE__ " / " __TIME__ "\n" );
	printf( " == Menu:\tL+[Enter] -> live\n" );
	printf( " ==      \tT+[Enter] -> teach\n" );
	printf( " ==      \tC+[Enter] -> calibrate (live image)\n" );
	printf( " ==      \tQ+[Enter] -> quit\n" );
	printf( " > " );
}

//----------------------------------------------------------------------
/* Check if the sample should be treminated. */
static bool check4Quit( const char *s )
{
	bool ret = true;
	if( strncmp( s, "Q", 1 ) == 0 || strncmp( s, "q", 1 ) == 0 )
		ret = false;
	return ret;
}

//----------------------------------------------------------------------
/* Check if the live mode should be started. */
static bool check4Live( const char *s )
{
	bool ret = false;
	if( strncmp( s, "L", 1 ) == 0 || strncmp( s, "l", 1 ) == 0 )
	{
		printf(" == " TOOL_PREFIX ": Running in live mode! Press any key to stop!\n" );

		while( !_kbhit() )
		{
			/* Grab a new image and update the display. */
			MYCHECKSDKCALL( DEV_Grab( deviceHandle, &imageHandle ));
			MYCHECKSDKCALL( DISP_UpdateWindow( hwnd, DISP_INVALIDATE ));
			MYCHECKSDKCALL( DISP_KeepRunning( NULL ));
		}

		ret = true;
	}

	return ret;
}

//----------------------------------------------------------------------
/* Check if the teach mode should be started. */
static bool check4Teach( const char *s )
{
	bool ret = false, bQFlag = false;
	if( strncmp( s, "T", 1 ) == 0 || strncmp( s, "t", 1 ) == 0 )
	{
		printf(" == " TOOL_PREFIX ": Running in teach mode....\n" );

		while( !_kbhit() )
		{
			/* Grab a new image and start the teach mode. */
			MYCHECKSDKCALL( DEV_Grab( deviceHandle, &imageHandle ));
			if( !CC_TeachCalibrationByGrid( camcalHandle, imageHandle ))
				printf("*** " TOOL_PREFIX ": %s\n", IPL_GetErrorString(IPL_GetErrorCode2()));
			else
			{
				/* Transform/calibrate the grabbed image and update the display.
				 * The display will show the grabbed image and not the calibrated image.
				 */
				MYCHECKSDKCALL( CC_TrCalibratedImage( camcalHandle, imageHandle, &ccImageHandle ));
				//IPL_PtBufCopy( ccImageHandle, &imageHandle, 0xffff, 0xffff, 0 );
				MYCHECKSDKCALL( DISP_UpdateWindow( hwnd, DISP_INVALIDATE ));
				MYCHECKSDKCALL( DISP_KeepRunning( NULL ));

				/* Read the statistic data. */
				CC_STATISTICS stat;
				MYCHECKSDKCALL( CC_Inquire( camcalHandle, CC_RO_STATISTICS, &stat ));

				printf("xavg: %8.3e xstddev: %8.3e\nyavg: %8.3e ystddev: %8.3e\ndavg: %8.3e dstddev: %8.3e\n",
							stat.dxavg, stat.xstddev, stat.dyavg, stat.ystddev, stat.euclavg, stat.euclstddev );
			}

			printf(" == " TOOL_PREFIX ": Teach again? (y/n/q)\n > " );

			char ls[3];
			memset( ls, 0, 3 );
			gets( ls );

			//printf(" gets ->>%s<<\n", ls );

			if( strncmp( ls, "N", 1 ) == 0 || strncmp( ls, "n", 1 ) == 0 )
				break;
			if( !check4Quit( ls ))
			{
				bQFlag = true;
				break;
			}
		}

		if( !bQFlag )
		{
			printf(" == " TOOL_PREFIX ": write data to the file \"%s\"!\n", pCalFlName );
			MYCHECKSDKCALL( CC_WriteToFile( camcalHandle, pCalFlName ));
		}

		ret = true;
	}

	return ret;
}

//----------------------------------------------------------------------
/* Check if the transformation/calibration mode should be started. */
static bool check4Calibrate( const char *s )
{
	bool ret = false;
	if( strncmp( s, "C", 1 ) == 0 || strncmp( s, "c", 1 ) == 0 )
	{
		printf(" == " TOOL_PREFIX ": Running in calibration mode! Press any key to stop!\n" );

		/* Remove existing calibration handel. */
		if( camcalHandle != IPL_DONT_CARE )
		{
			CC_BufFree( camcalHandle );
			camcalHandle = IPL_DONT_CARE;
		}

		/* Try to read the data from the calibration file. */
		if( !CC_ReadFromFile( &camcalHandle, pCalFlName ))
		{
			if( IPL_GetErrorCode2() == CC_ERR_NO_FILE_FOUND )
			{
				printf(" == " TOOL_PREFIX ": Please run first teach!\n" );

				/* realloc the buffer which was previous deleted */
				MYCHECKSDKCALL( CC_BufAlloc( &camcalHandle, 0 ));
			}
			else
				printf("*** " TOOL_PREFIX ": %s\n", IPL_GetErrorString(IPL_GetErrorCode2()));
			return ret;
		}


		while( !_kbhit() )
		{
			/* Transform/calibrate the grabbed image and update the display.
			 * The calibrated image will be displayed now.
			 */
			MYCHECKSDKCALL( DEV_Grab( deviceHandle, &imageHandle ));
			MYCHECKSDKCALL( CC_TrCalibratedImage( camcalHandle, imageHandle, &ccImageHandle ));

			IPL_PtBufCopy( ccImageHandle, &imageHandle, 0xffff, 0xffff, 0 );

			MYCHECKSDKCALL( DISP_UpdateWindow( hwnd, DISP_INVALIDATE ));
			MYCHECKSDKCALL( DISP_KeepRunning( NULL ));
		}

		ret = true;
	}

	return ret;
}

//----------------------------------------------------------------------
int main( int /*argc*/, char** /*argv[]*/ )
{
	/* Alloc a calibration buffer.
	 * The default settings are made.
	 */
	MYCHECKSDKCALL( CC_BufAlloc( &camcalHandle, 0 ));

	/* Open and initialize a device.
	 * Grab also a image.
	 */
	MYCHECKSDKCALL( DEV_Open( &deviceHandle, "matrixvision.ddc" ));
    MYCHECKSDKCALL( DEV_Control( deviceHandle, DEV_SINGLE, 0 ));
	MYCHECKSDKCALL( DEV_Grab( deviceHandle, &imageHandle ));

	/* Open the display window with the grabbed image. */
	MYCHECKSDKCALL( DISP_OpenWindow( imageHandle, &hwnd, 0, NULL ));
	MYCHECKSDKCALL( DISP_UpdateWindow( hwnd, DISP_INVALIDATE ));
	MYCHECKSDKCALL( DISP_KeepRunning( NULL ));

	/* Start the main loop. */
	bool bRunFlag = true;
	while( bRunFlag )
	{
		schowMenu( );

		char s[3];
		memset( s, 0, 3 );
		gets( s );
		dprintf((" == " TOOL_PREFIX ": %s - >>%s<<\n", __FUNCTION__, s ));

		/* Read the stdin. */
		if(( bRunFlag = check4Quit( s )))
		{
			if( !check4Live( s ))
			{
				if( !check4Teach( s ))
					check4Calibrate( s );
			}
		}

		/* Grab a new image and update the display. */
		MYCHECKSDKCALL( DEV_Grab( deviceHandle, &imageHandle ));
		MYCHECKSDKCALL( DISP_UpdateWindow( hwnd, DISP_INVALIDATE ));
		MYCHECKSDKCALL( DISP_KeepRunning( NULL ));
	}

	printf(" ==\n == " TOOL_PREFIX ": CU.... :o)\n\n" );
	FreeAllHandles( );

	return 0;
}

//----------------------------------------------------------------------
//----------------------------------------------------------------------



