//--------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------
/*
* This is a sample how to use the GMM module.
* It demonstrates all basic functions using two images "model.bmp" and "scene.bmp".
*
* 1. Load model image
* 2. Load scene image
* 3. Create model description
* 4. Show selected corners and arcs
* 5. Set options for pattern matching
* 6. Run pattern matching
* 7. Show resulting transformations
*
*/

//--------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------
#include <stdio.h>
#include <math.h>

#ifndef M_PI
#define M_PI       3.14159265358979323846
#endif

#include <system.h>

#include <vl_base.h>
#include <vl_file.h>
#include <vl_gmm.h>
#include <vl_display.h>

//--------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------
#define CHECK_RESULT(X) \
	if ((X) == IPL_FALSE) \
	{ \
		printf("Statement failed:" #X "\n"); \
		translateError(IPL_GetErrorCode()); \
		abort(); \
	}

//--------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------
static IPL_HWND hwnd = 0;
static DISP_OOISETHANDLE oois = DISP_DONT_CARE;
static DISP_OOIHANDLE *loi1 = NULL, *loi2 = NULL, *poi = NULL;
static int ooiCnt = 0;
static IPL_DOUBLE dx = 0.0, dy = 0.0;

void translateError        (IPL_FRESULT err);
void printSelectedFeatures (GMM_MODELHANDLE hModel);
void setOptions            (GMM_MATCHERHANDLE hMather);
void printMatches          (GMM_MATCH* pMatches, IPL_ULONG nCountMatches);
void draw_corner           (const GMM_CORNER corner, const int index );
void draw_results          (const GMM_MATCH Matches, const int index );
void delete_lines          ();
void mk_new_lines          ();

//--------------------------------------------------------------------------------------------
int main( int /*argc*/, char** /*argv[]*/)
{
	GMM_MODELHANDLE   hMyModel;
	GMM_MODELHANDLE   hModel;
	GMM_MATCHERHANDLE hMatcher;
	IPL_BUFHANDLE     hEdgePoints;
	IPL_BUFHANDLE     hModelImage;
	IPL_BUFHANDLE     hSceneImage;

	// Load model image
	hModelImage = IPL_DONT_CARE;
	CHECK_RESULT (IPL_FlImport (&hModelImage, NULL, FL_DONT_CARE, "model.bmp"));

	// display the loaded image
	CHECK_RESULT (DISP_OpenWindow( hModelImage, &hwnd, 0, NULL));
	CHECK_RESULT (DISP_KeepRunning("The Model! Click the space bar to continue\n"));

	//
	CHECK_RESULT (DISP_OoiSetAlloc( &oois ));

	// Model
	CHECK_RESULT (GMM_ModelAlloc(&hModel));
	CHECK_RESULT (GMM_ModelCreateFromImage(hModel, hModelImage, 7, 30));

	// Read edge points of model
	hEdgePoints = IPL_DONT_CARE;
	CHECK_RESULT (GMM_ModelGetEdgePoints (hModel, &hEdgePoints));

	// Print all model features:
	printSelectedFeatures(hModel);

	// display the corners
	DISP_ControlWindow( DISP_OOISET, hwnd, &oois );
	DISP_UpdateWindow( hwnd, DISP_INVALIDATE );
	DISP_KeepRunning("The 7 corners! Click the space bar to continue\n");
	delete_lines( );

	// display the edge points
	DISP_OpenWindow( hEdgePoints, &hwnd, 0, NULL);
	DISP_KeepRunning("The Edge Points! Click the space bar to continue\n");

	///////////////////////////////////////////////////////////////////////////
	// create new model with selected corners, display and save it to file
	///////////////////////////////////////////////////////////////////////////

	///////////////////////////////////////////////////////////////////////////
	// create new model with selected corners from the original model

	// Allocate empty New Model (hMyModel)
	// with same width and height of original Model:
	CHECK_RESULT (GMM_ModelAlloc(&hMyModel));
	IPL_ULONG width; CHECK_RESULT (GMM_ModelInquire(hModel, MODEL_WIDTH, &width));
	IPL_ULONG height; CHECK_RESULT (GMM_ModelInquire(hModel, MODEL_HEIGHT, &height));
	CHECK_RESULT (GMM_ModelCreateUserDefined(hMyModel, width, height));

	// Set edge map of original model to new model.
	CHECK_RESULT (GMM_ModelSetEdgePoints (hMyModel, hEdgePoints));
	CHECK_RESULT (IPL_BufFree(hEdgePoints));

	// Copy the first 5 (or all, if there are 5 or less) corners to new structure

	IPL_ULONG          nCountCorners;
	CHECK_RESULT (GMM_ModelCountCorners(hModel, &nCountCorners));

	IPL_ULONG nCountCopyCorners = nCountCorners;
	if(nCountCopyCorners > 5) { nCountCopyCorners = 5; }
	// if(0 == nCountCopyCorners) { error(); } // !!!!!!

	// get the list of corner handles
	GMM_FEATUREHANDLE* pCornerHandleList = new GMM_FEATUREHANDLE[nCountCorners];
	CHECK_RESULT (GMM_ModelGetCornerHandles(hModel, pCornerHandleList));

	// do the copy:
	for (IPL_ULONG i = 0; i < nCountCopyCorners; i++)
	{
		GMM_CORNER  corner;
		// read corner from original model
		CHECK_RESULT (GMM_ModelGetCorner(hModel, pCornerHandleList[i], &corner));
		// add corner to new model
		CHECK_RESULT (GMM_ModelAddCorner(hMyModel, &corner, NULL));
		// note:
		// handle of new corner is here not of interest, so pass NULL instead of
		// a pointer to a handle.
	}
	delete[] pCornerHandleList;

	///////////////////////////////////////////////////////////////////////////
	// print new model

	// Print selected 5 model features from copied model:
	printSelectedFeatures(hMyModel);

	// display the corners
	CHECK_RESULT (DISP_OpenWindow( hModelImage, &hwnd, 0, NULL));
	DISP_ControlWindow( DISP_OOISET, hwnd, &oois );
	DISP_UpdateWindow( hwnd, DISP_INVALIDATE );
	DISP_KeepRunning("The 5 corners! Click the space bar to continue\n");
	delete_lines( );

	///////////////////////////////////////////////////////////////////////////
	// save new model to file

	// Save copied model to file "copiedmodel.gmm"
	CHECK_RESULT (GMM_ModelSave(hMyModel, "copiedmodel.gmm"));


	// free new model, since we don't need it anymore in this file (for now):
	CHECK_RESULT (GMM_ModelFree(hMyModel));

	///////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////



	// Set rotation center to the middle of the model image
	dx = 0.0;
	dy = 0.0;

	// Define x coordinate of the rotation center (Default 0)
	IPL_ULONG nModelWidth;
	GMM_ModelInquire(hModel, MODEL_WIDTH, &nModelWidth);
	dx = (IPL_DOUBLE)nModelWidth/2.0;
	CHECK_RESULT (GMM_ModelControl(hModel, MODEL_ROTATION_CENTER_X, &dx));

	// Define y coordinate of the rotation center (Default 0)
	IPL_ULONG nModelHeight;
	GMM_ModelInquire(hModel, MODEL_HEIGHT, &nModelHeight);
	dy = (IPL_DOUBLE)nModelHeight/2.0;
	CHECK_RESULT (GMM_ModelControl(hModel, MODEL_ROTATION_CENTER_Y, &dy));

	// Get a new pattern matcher handle
	CHECK_RESULT (GMM_MatcherAlloc(&hMatcher));

	// Set options for pattern matching. It is not obligatory to set options.
	// You can also use default parameters.
	setOptions(hMatcher);

	// Load scene image
	hSceneImage = IPL_DONT_CARE;
	CHECK_RESULT (IPL_FlImport (&hSceneImage, NULL, FL_DONT_CARE, "scene.bmp"));

	// Start pattern matching
	CHECK_RESULT (GMM_MatcherFindModel(hMatcher, hModel, hSceneImage));

	// Get matches
	GMM_MATCH* pMatches;
	IPL_ULONG  nCountMatches;
	CHECK_RESULT(GMM_MatcherGetMatches(hMatcher, &pMatches, &nCountMatches));

	// Print matches
	printMatches(pMatches, nCountMatches);

	// display the scene image
	DISP_OpenWindow( hSceneImage, &hwnd, 0, NULL);
	DISP_ControlWindow( DISP_OOISET, hwnd, &oois );
	DISP_KeepRunning("The Scene! Click the space bar to continue\n");
	delete_lines( );

	// Free
	CHECK_RESULT (GMM_ModelFree(hModel));
	if (nCountMatches > 0)
	{
		CHECK_RESULT (GMM_MatcherFreeMatches(hMatcher, pMatches));
	}
	CHECK_RESULT (GMM_MatcherFree(hMatcher));

	CHECK_RESULT (IPL_BufFree(hSceneImage));
	CHECK_RESULT (IPL_BufFree(hModelImage));

	return 0;
}

//--------------------------------------------------------------------------------------------
void printSelectedFeatures(GMM_MODELHANDLE hModel)
{
	dprintf(("\nFeatures that are used for pattern matching:\n"));

	// ----- Corners ----- //
	IPL_ULONG          nCountCorners;
	GMM_FEATUREHANDLE* pCornerList;

	CHECK_RESULT (GMM_ModelCountCorners(hModel, &nCountCorners));
	dprintf(("\nNumber of corners %ld\n", nCountCorners));

	if (nCountCorners > 0)
	{
		pCornerList = new GMM_FEATUREHANDLE[nCountCorners];
		CHECK_RESULT (GMM_ModelGetCornerHandles(hModel, pCornerList));

		/* ++ used to display the corner pts ++ */
		ooiCnt = nCountCorners;
		delete_lines( );
		mk_new_lines( );
		/* -- used to display the corner pts -- */

		GMM_CORNER  corner;
		for (IPL_ULONG i = 0; i < nCountCorners; i++)
		{
			CHECK_RESULT (GMM_ModelGetCorner(hModel, pCornerList[i], &corner));
			dprintf(("Corner %3ld: Corner point: %3d/%3d, opening angle: %3d\n", i, (int)corner.x, (int)corner.y, (int)(corner.openingAngle/M_PI*180.0f)));
			draw_corner( corner, i );
		}

		delete[] pCornerList;
	}

	// ----- Arcs ----- //
	IPL_ULONG          nCountArcs;
	GMM_FEATUREHANDLE* pArcList;

	CHECK_RESULT (GMM_ModelCountArcs(hModel, &nCountArcs));
	dprintf(("\nNumber of arcs %ld\n", nCountArcs));

	if (nCountArcs > 0)
	{
		pArcList = new GMM_FEATUREHANDLE[nCountArcs];
		CHECK_RESULT (GMM_ModelGetArcHandles(hModel, pArcList));

		IPL_ULONG   i;
		GMM_ARC     arc;

		for (i = 0; i < nCountArcs; i++)
		{
			CHECK_RESULT (GMM_ModelGetArc(hModel, pArcList[i], &arc));
			dprintf(("Arc %ld: \n", i));
			dprintf(("Center point: %d/%d, radius: %d\n", (int)arc.x, (int)arc.y, (int)arc.radius));
		}

		delete[] pArcList;
	}
}

//--------------------------------------------------------------------------------------------
void setOptions(GMM_MATCHERHANDLE hMatcher)
{
	IPL_DOUBLE f;
	IPL_LONG  n;

	// Minimum score to accept a match (Default 0.7)
	// You should reduce it the image quality is bad or
	// if the searched object is occluded.
	f = 0.8f;
	CHECK_RESULT (GMM_MatcherControl(hMatcher, MATCHER_MIN_SCORE,  &f));

	n = GMM_MATCHER_FIND_ALL;
	CHECK_RESULT (GMM_MatcherControl(hMatcher, MATCHER_NUMBER_OF_MATCHES, &n));
}

//--------------------------------------------------------------------------------------------
void printMatches(GMM_MATCH *pMatches, IPL_ULONG nCountMatches)
{
	dprintf(("\nNumber of matches: %ld\n", nCountMatches));

	/* ++ used to display the corner pts ++ */
	ooiCnt = nCountMatches;
	delete_lines( );
	mk_new_lines( );
	poi = new DISP_OOIHANDLE[ooiCnt];
	memset( poi, IPL_DONT_CARE, sizeof( DISP_OOIHANDLE ) * ooiCnt );
	IPL_POINT Pt;
	IPL_COLOR Color = PALETTERGB( 0,255,255 );
	char msg[512];
	/* -- used to display the corner pts -- */

	for (IPL_ULONG i = 0; i < nCountMatches; i++)
	{
		dprintf(("\nMatch %ld: \n", i));
		dprintf(("x translation: %3.2f\n", pMatches[i].x));
		dprintf(("y translation: %3.2f\n", pMatches[i].y));
		dprintf(("rotation:      %3.2f\n", (pMatches[i].r/M_PI*180.0f)));
		dprintf(("scaling:       %3.2f\n", pMatches[i].s));
		dprintf(("score:         %3.2f\n", pMatches[i].score));
		draw_results( pMatches[i], i );

		/* ++ used to display the result text ++ */
		Pt.xpos = 200;
		Pt.ypos = 20*i + 20;
		poi[i] = DISP_DONT_CARE;
		memset(msg, 0, 512);
		sprintf( msg, "%ld - tx: %3.2f ty: %3.2f rot: %3.2f scal: %3.2f q: %3.2f%%",
			i, pMatches[i].x, pMatches[i].y, (pMatches[i].r/M_PI*180.0f), pMatches[i].s, pMatches[i].score*100. );

		DISP_OoiAlloc( oois, &poi[i], DISP_POI_TOOL, &Pt, 1, NULL );
		IPL_BOOL Visible=IPL_FALSE;
		DISP_ControlOoi( oois, poi[i], DISP_OOI_LINES_VISIBLE, &Visible );
		DISP_ControlOoi( oois, poi[i], DISP_OOI_TEXT, msg );
		DISP_ControlOoi( oois, poi[i], DISP_OOI_TEXT_COLOR, &Color );
		/* -- used to display the result text -- */
	}
}

//--------------------------------------------------------------------------------------------
#define WRITE_CASE(CONSTANT) \
	case CONSTANT: printf("Error: " #CONSTANT " (%d)\n", (int)err); break

//--------------------------------------------------------------------------------------------
void translateError(IPL_FRESULT err)
{
	switch (err)
	{
		WRITE_CASE(IPL_NOT_SPECIFIED);
		WRITE_CASE(IPL_MEMORY_ERROR);
		WRITE_CASE(IPL_ALLOCATION_ERROR);
		WRITE_CASE(IPL_HANDLE_ERROR);
		WRITE_CASE(IPL_INVALID_PARAM_ERROR);
		WRITE_CASE(GMM_WRONG_IMAGE_SIZE);
		WRITE_CASE(GMM_OUT_OF_BOUNDS);
		WRITE_CASE(GMM_INTERNAL_ERROR);
		WRITE_CASE(GMM_NO_EIGHT_BIT_IMAGE);
		WRITE_CASE(GMM_NO_MODEL_IMAGE);
		WRITE_CASE(GMM_MODEL_NOT_READY);
		WRITE_CASE(GMM_FILE_NOT_FOUND);
		WRITE_CASE(GMM_FEATURE_NOT_FOUND);
		WRITE_CASE(GMM_IMAGE_TOO_SMALL);
		WRITE_CASE(GMM_MODEL_DESCRIPTION_CORRUPT);
		WRITE_CASE(GMM_BAD_OPTION_KEY);
		WRITE_CASE(GMM_MODEL_NOT_CREATED);
		WRITE_CASE(GMM_INVALID_CONTROL_PARAM);
		WRITE_CASE(GMM_READ_ONLY_PARAMETER);
		default: printf("Unknown error: %d\n", (int)err); break;
	}
}

//--------------------------------------------------------------------------------------------
void draw_corner( const GMM_CORNER corner, const int index )
{
	if( index >= 0 && index < ooiCnt )
	{
		IPL_POINT LPts[2];
		IPL_COLOR Color = PALETTERGB( 255,0,0 );

		LPts[0].xpos = (int)corner.x - 5;
		LPts[0].ypos = (int)corner.y;
		LPts[1].xpos = (int)corner.x + 5;
		LPts[1].ypos = (int)corner.y;
		loi1[index] = DISP_DONT_CARE;
		DISP_OoiAlloc( oois, &loi1[index], DISP_LOI_TOOL, LPts, sizeof( LPts)/sizeof( LPts[0] ), NULL );
		DISP_ControlOoi( oois, loi1[index], DISP_OOI_COLOR, &Color );

		LPts[0].xpos = (int)corner.x;
		LPts[0].ypos = (int)corner.y - 5;
		LPts[1].xpos = (int)corner.x;
		LPts[1].ypos = (int)corner.y + 5;
		loi2[index] = DISP_DONT_CARE;
		DISP_OoiAlloc( oois, &loi2[index], DISP_LOI_TOOL, LPts, sizeof( LPts)/sizeof( LPts[0] ), NULL );
		DISP_ControlOoi( oois, loi2[index], DISP_OOI_COLOR, &Color );
	}
}

//--------------------------------------------------------------------------------------------
void draw_results( const GMM_MATCH Matches, const int index )
{
	if( index >= 0 && index < ooiCnt )
	{
		IPL_POINT LPts[2];
		IPL_COLOR Color = PALETTERGB( 0,255,0 );

		LPts[0].xpos = (int)( dx + Matches.x - 5 );
		LPts[0].ypos = (int)( dy + Matches.y );
		LPts[1].xpos = (int)( dx + Matches.x + 5 );
		LPts[1].ypos = (int)( dy + Matches.y );
		loi1[index] = DISP_DONT_CARE;
		DISP_OoiAlloc( oois, &loi1[index], DISP_LOI_TOOL, LPts, sizeof( LPts)/sizeof( LPts[0] ), NULL );
		DISP_ControlOoi( oois, loi1[index], DISP_OOI_COLOR, &Color );

		LPts[0].xpos = (int)( dx + Matches.x );
		LPts[0].ypos = (int)( dy + Matches.y - 5);
		LPts[1].xpos = (int)( dx + Matches.x );
		LPts[1].ypos = (int)( dy + Matches.y + 5 );
		loi2[index] = DISP_DONT_CARE;
		DISP_OoiAlloc( oois, &loi2[index], DISP_LOI_TOOL, LPts, sizeof( LPts)/sizeof( LPts[0] ), NULL );
		DISP_ControlOoi( oois, loi2[index], DISP_OOI_COLOR, &Color );

		Color = PALETTERGB( 0,0,255 );
		char msg[80];
		sprintf( msg, "%d", index);
		DISP_ControlOoi( oois, loi2[index], DISP_OOI_TEXT, msg );
		DISP_ControlOoi( oois, loi2[index], DISP_OOI_TEXT_COLOR, &Color );
	}
}

//--------------------------------------------------------------------------------------------
void delete_lines( )
{
	if( loi1 != NULL )
	{
		for( int i = 0;  i < ooiCnt; i++ )
		{
			if( loi1[i] != DISP_DONT_CARE )
			{
				DISP_OoiFree( oois, loi1[i] );
				loi1[i] = DISP_DONT_CARE;
			}
		}

		delete loi1;
		loi1 = NULL;
	}

	if( loi2 != NULL )
	{
		for( int i = 0;  i < ooiCnt; i++ )
		{
			if( loi2[i] != DISP_DONT_CARE )
			{
				DISP_OoiFree( oois, loi2[i] );
				loi2[i] = DISP_DONT_CARE;
			}
		}

		delete loi2;
		loi2 = NULL;
	}

	if( poi != NULL )
	{
		for( int i = 0;  i < ooiCnt; i++ )
		{
			if( poi[i] != DISP_DONT_CARE )
			{
				DISP_OoiFree( oois, poi[i] );
				poi[i] = DISP_DONT_CARE;
			}
		}

		delete poi;
		poi = NULL;
	}
}

//--------------------------------------------------------------------------------------------
void mk_new_lines( )
{
	if( ooiCnt > 0 )
	{
		loi1 = new DISP_OOIHANDLE[ooiCnt];
		memset( loi1, IPL_DONT_CARE, sizeof( DISP_OOIHANDLE ) * ooiCnt );

		loi2 = new DISP_OOIHANDLE[ooiCnt];
		memset( loi2, IPL_DONT_CARE, sizeof( DISP_OOIHANDLE ) * ooiCnt );
	}
}

//--------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------
