/**
\xmlonly
mvIMPACT Data Matrix
\endxmlonly

\page datamatrix_page Datamatrix example (CPP)


\section datamatrix_section_purpose Purpose
This example shows how to read a datamatrix code in an image.

\section datamatrix_section_intro What it does
This example shows how to locate and decode a datamatrix code in an image and print the
result as well as some statistics.

\section datamatrix_section_description How it works
First the input image is loaded and displayed.

\code
		//load and diplay image
		Image image = loadImage("dmtrx_6.bmp");
		Window window = Window(image);
\endcode

Then a new DatamatrixConfiguration object is created and some of the settings are
changed.

\code
		//setup datamatrix configuration
		DatamatrixConfiguration config = DatamatrixConfiguration();
		config.setSpacing(20); 
		config.setContrast(50); 
\endcode

Afterwards the decoding process is started.

\code
		//decode image
		DatamatrixResults results = decodeDM(image,config);

\endcode

Then some general statistics about the decoding process are displayed:

\code
		//get candidates and decoding time
		int candidates = results.getNumberOfCandidates();
		printf("Candidates %i\n", results.getNumberOfCandidates());
		printf("Total decoding time: %3.2f ms\n",results.getTotalDecodingTime());

\endcode

Then for each found candidate some statistics are displayed. Afterwards
each code type which produces sensible results is displayed for the current
code.

\code
		//step through all candidates and display information
		for (int candCounter=0; candCounter < candidates;candCounter++) {
			printf("\tCandidate\t%i:\n",candCounter);
			printf("\t\tDecoding time:\t%3.2f ms\n",results.getDecodingTime(candCounter));
			int codes = results.getNumberOfCodes(candCounter);

			//setp through all codes and display information
			for (int codeCounter=0;codeCounter < codes; codeCounter++) {

				//get information
				std::string codeType = results.getCodeType(candCounter,codeCounter);
				std::string code = results.getCode(candCounter,codeCounter);
				Size dimensions = results.getDatamatrixDims(candCounter,codeCounter);
				int errors = results.getNumberOfErrors(candCounter,codeCounter);
				double score = results.getReadabilityScore(candCounter,codeCounter);
				bool flipped = results.getFlipped(candCounter, codeCounter);

				//print information
				printf("\t\tCode:\t\t%i\n",codeCounter);
				printf("\t\tCodetype:\t%s\n",codeType.c_str());
				printf("\t\tDimensions:\t%i x %i\n",dimensions.getWidth(), dimensions.getHeight());
				if (!flipped)
					printf("\t\tFlipped: \tfalse, code seen from front\n");
				else
					printf("\t\tFlipped: \ttrue, code seen from back\n");
				printf("\t\tErrors:\t\t%i\n",errors);
				printf("\t\tScore:\t\t%f\n",score);
				printf("\t\tData:\t\t%s\n",code.c_str());
			}
		}
\endcode

Finally the sample waits for space to be pressed.

\code
		//keep sample alive		
		runMessageLoop();
\endcode

\section datamatrix_section_source Source file

<matrixdoc>
*/

#include "mvIMPACT.h"
#include "mvIMPACT_datamatrix.h"


int main(int , char* [])
{
	try
	{

		//load and diplay image
		Image image = loadImage("dmtrx_6.bmp");
		Window window = Window(image);

		//setup datamatrix configuration
		DatamatrixConfiguration config = DatamatrixConfiguration();
		config.setSpacing(20); 
		config.setContrast(50); 
		config.setCheckPrintQuality(true);

		//decode image
		DatamatrixResults results = decodeDM(image,config);

		//get candidates and decoding time
		int candidates = results.getNumberOfCandidates();
		printf("Candidates %i\n", results.getNumberOfCandidates());
		printf("Total decoding time: %3.2f ms\n",results.getTotalDecodingTime());

		//step through all candidates and display information
		for (int candCounter=0; candCounter < candidates;candCounter++) {
			printf("\tCandidate\t%i:\n",candCounter);
			printf("\t\tDecoding time:\t%3.2f ms\n",results.getDecodingTime(candCounter));
			int codes = results.getNumberOfCodes(candCounter);

			//setp through all codes and display information
			for (int codeCounter=0;codeCounter < codes; codeCounter++) {

				//get information
				std::string codeType = results.getCodeType(candCounter, codeCounter);
				std::string code = results.getCode(candCounter, codeCounter);
				Size dimensions = results.getDatamatrixDims(candCounter, codeCounter);
				int errors = results.getNumberOfErrors(candCounter, codeCounter);
				double score = results.getReadabilityScore(candCounter, codeCounter);
				bool flipped = results.getFlipped(candCounter, codeCounter);

				long rmin = results.getReflectanceMinimum(candCounter);
				long rmax = results.getReflectanceMaximum(candCounter);
				double rdark = results.getReflectanceDark(candCounter);
				double rbright = results.getReflectanceBright(candCounter);
				double sc = results.getSymbolContrast(candCounter);
				double mod = results.getModulation(candCounter);
				double fpd = results.getFixedPatternDamage(candCounter);
				double anu = results.getAxialNonuniformity(candCounter);
				double gnu = results.getGridNonuniformity(candCounter);
				double uec = results.getUnusedErrorCorrection(candCounter);
				double decg = results.getDecodabilityGrade(candCounter);
				double scg = results.getSymbolContrastGrade(candCounter);
				double modg = results.getModulationGrade(candCounter);
				double fpdg = results.getFixedPatternDamageGrade(candCounter);
				double anug = results.getAxialNonuniformityGrade(candCounter);
				double gnug = results.getGridNonuniformityGrade(candCounter);
				double uecg = results.getUnusedErrorCorrectionGrade(candCounter);
				double sg = results.getScanGrade(candCounter);

				//print information
				printf("\t\tCode:\t\t%i\n",codeCounter);
				printf("\t\tCodetype:\t%s\n",codeType.c_str());
				printf("\t\tDimensions:\t%i x %i\n",dimensions.getWidth(), dimensions.getHeight());
				if (!flipped)
					printf("\t\tFlipped: \tfalse, code seen from front\n");
				else
					printf("\t\tFlipped: \ttrue, code seen from back\n");
				printf("\t\tErrors:\t\t%i\n",errors);
				printf("\t\tScore:\t\t%f\n",score);
				printf("\t\tData:\t\t%s\n",code.c_str());
				printf("\t\tPrint quality:\n");
				printf("\t\t\tISO/IEC 15415 Reflectance minimum: %ld\n", rmin);
				printf("\t\t\tISO/IEC 15415 Reflectance maximum: %ld\n", rmax);
				printf("\t\t\tISO/IEC 15415 Dark reflectance: %.3f\n", rdark);
				printf("\t\t\tISO/IEC 15415 Bright reflectance: %.3f\n", rbright);
				printf("\t\t\tISO/IEC 15415 Symbol contrast: %.3f\n", sc);
				printf("\t\t\tISO/IEC 15415 Modulation: %.3f\n", mod);
				printf("\t\t\tISO/IEC 15415 Fixed pattern damage: %.3f\n", fpd);
				printf("\t\t\tISO/IEC 15415 Axial nonuniformity: %.3f\n", anu);
				printf("\t\t\tISO/IEC 15415 Grid nonuniformity: %.3f\n", gnu);
				printf("\t\t\tISO/IEC 15415 Unused error correction: %.3f\n", uec);
				printf("\t\t\tISO/IEC 15415 Decodability grade: %.1f", decg);
				if (decg >= 3.5)
					printf("(A)\n");
				else if (decg >= 2.5)
					printf("(B)\n");
				else if (decg >= 1.5)
					printf("(C)\n");
				else if (decg >= 0.5)
					printf("(D)\n");
				else
					printf("(F)\n");
				printf("\t\t\tISO/IEC 15415 Symbol contrast grade: %.1f", scg);
				if (scg >= 3.5)
					printf("(A)\n");
				else if (scg >= 2.5)
					printf("(B)\n");
				else if (scg >= 1.5)
					printf("(C)\n");
				else if (scg >= 0.5)
					printf("(D)\n");
				else
					printf("(F)\n");
				printf("\t\t\tISO/IEC 15415 Modulation grade: %.1f", modg);
				if (modg >= 3.5)
					printf("(A)\n");
				else if (modg >= 2.5)
					printf("(B)\n");
				else if (modg >= 1.5)
					printf("(C)\n");
				else if (modg >= 0.5)
					printf("(D)\n");
				else
					printf("(F)\n");
				printf("\t\t\tISO/IEC 15415 Fixed pattern damage grade: %.1f", fpdg);
				if (fpdg >= 3.5)
					printf("(A)\n");
				else if (fpdg >= 2.5)
					printf("(B)\n");
				else if (fpdg >= 1.5)
					printf("(C)\n");
				else if (fpdg >= 0.5)
					printf("(D)\n");
				else
					printf("(F)\n");
				printf("\t\t\tISO/IEC 15415 Axial Nonuniformity grade: %.1f", anug);
				if (anug >= 3.5)
					printf("(A)\n");
				else if (anug >= 2.5)
					printf("(B)\n");
				else if (anug >= 1.5)
					printf("(C)\n");
				else if (anug >= 0.5)
					printf("(D)\n");
				else
					printf("(F)\n");
				printf("\t\t\tISO/IEC 15415 Grid Nonuniformity grade: %.1f", gnug);
				if (gnug >= 3.5)
					printf("(A)\n");
				else if (gnug >= 2.5)
					printf("(B)\n");
				else if (gnug >= 1.5)
					printf("(C)\n");
				else if (gnug >= 0.5)
					printf("(D)\n");
				else
					printf("(F)\n");
				printf("\t\t\tISO/IEC 15415 Unused error correction grade: %.1f", uecg);
				if (uecg >= 3.5)
					printf("(A)\n");
				else if (uecg >= 2.5)
					printf("(B)\n");
				else if (uecg >= 1.5)
					printf("(C)\n");
				else if (uecg >= 0.5)
					printf("(D)\n");
				else
					printf("(F)\n");
				printf("\t\t\tISO/IEC 15415 Scan grade: %.1f", sg);
				if (sg >= 3.5)
					printf("(A)\n");
				else if (sg >= 2.5)
					printf("(B)\n");
				else if (sg >= 1.5)
					printf("(C)\n");
				else if (sg >= 0.5)
					printf("(D)\n");
				else
					printf("(F)\n");
			}
		}

		//keep sample alive		
		runMessageLoop();

	}
	catch(ImpactException& e)
	{
		std::cerr << e.getErrorString() << std::endl;
	}

	return 0;
}

//</matrixdoc>
