/**
\xmlonly
mvIMPACT Base
\endxmlonly

\page morph_page Morphology example (CPP)

\section morph_section_purpose Purpose
This sample shows, how to use morphologic operations.

\section morph_section_intro What it does
This sample shows how to use different morphologic operations and how to setup and use
user defined structuring elements.

\section morph_section_description How it works
First of all an image is loaded into memory and a window is created to show
the image.

\code
    // Create a new ImageBuffer
    Image sourceImage = loadImage("can.bmp");

    // Display an image in a new window and wait until the user presses a key
    Window sourceWindow = Window(sourceImage);
    runMessageLoop("Press space to erode the image (3x3 squared)\n");

\endcode


Then the source image is eroded and stored in new image. The new image is displayed in a new window.
The erosion uses default parameters here, which means, that the default structuring element is used and
that only one iteration is done.

\code
    // Erode an image using the default structuring element
    Image image2 = erode(sourceImage);

    // Display an image in a new window and wait until the user presses a key
    Window destWindow = Window(image2);
    runMessageLoop("Press space to dilate the image (3x3 cross)\n");
\endcode

After the user pressed the space key the first image is dilated using a predefined structuring element.
The resulting image is displayed in the window that was created in the step before. This is done
by exchanging the image buffers displayed in the window.

\code
	// Dilate an image using a predefined structuring element
    Image image3 = dilate(sourceImage,sseCross);

    // Display an image in an existing window
    destWindow.setBuffer(image3);
    destWindow.update();
    runMessageLoop("Press space to open the image (5x5 cross)\n");

\endcode

In the next step a user-defined 5x5 structuring element is used to do an opening operation on the source image.
First the bits of the structuring element are set in an array. Then the structuring element is created
and the bits are set according to the bits set in the array. After the bits are set in the structuring element
the open operation can be started. The result of the operation is shown in the result window.
\code
	// Create a 5x5 structuring element that has the shape of a cross
    const int cross_5x5[] =
      {0,0,1,0,0,
       0,0,1,0,0,
       1,1,1,1,1,
       0,0,1,0,0,
       0,0,1,0,0};
    StructuringElement se1 = StructuringElement(5,5);
    se1.setValues(std::vector<double>(cross_5x5,cross_5x5+25));

    // Open an image with a user-defined structuring element
    Image image4 = opening(sourceImage,se1,1);

    // Display an image in an existing window
    destWindow.setBuffer(image4);
    destWindow.update();
    runMessageLoop("Press space to close the image (3x3 squared with center hole)\n");

\endcode

In the last step a 3x3 structuring element is created. This structuring element has the shape
of a rectangle with a hole in the center. Again, as already seen in the previous step, the bits
are first being set in an array and are then assigned to the entire structuring element inside the loops.
Then a closing operation is done on the source image using the created 3x3 structuring element.
Then the second window is updated, such that it shows the resulting image of the closing operation.

\code
	// Create a 3x3 structuring element that has the shape of a square with a hole in the center
    const int square_3x3_hole[] =
      {1,1,1,
       1,0,1,
       1,1,1};
    StructuringElement se2 = StructuringElement(3,3);
    se2.setValues(std::vector<double>(square_3x3_hole,square_3x3_hole+9));

    // Close an image using a user-defined structuring element
    Image image5 = closing(sourceImage,se2,1);

    // Display an image in an existing window
    destWindow.setBuffer(image5);
    destWindow.update();
    runMessageLoop("Press space to quit\n");

\endcode


\section morph_section_source Source file

<matrixdoc>
*/

#include "mvIMPACT.h"

int main(int , char*[] )
{
  try
  {
    // Create a new ImageBuffer
    Image sourceImage = loadImage("can.bmp");

    // Display an image in a new window and wait until the user presses a key
    Window sourceWindow = Window(sourceImage);
    runMessageLoop("Press space to erode the image (3x3 squared)\n");





    // Erode an image using the default structuring element
    Image image2 = erode(sourceImage);

    // Display an image in a new window and wait until the user presses a key
    Window destWindow = Window(image2);
    runMessageLoop("Press space to dilate the image (3x3 cross)\n");





	// Dilate an image using a predefined structuring element
    Image image3 = dilate(sourceImage,sseCross);

    // Display an image in an existing window
    destWindow.setBuffer(image3);
    destWindow.update();
    runMessageLoop("Press space to open the image (5x5 cross)\n");



	// Create a 5x5 structuring element that has the shape of a cross
    const int cross_5x5[] =
      {0,0,1,0,0,
       0,0,1,0,0,
       1,1,1,1,1,
       0,0,1,0,0,
       0,0,1,0,0};
    StructuringElement se1 = StructuringElement(5,5);
    se1.setValues(std::vector<double>(cross_5x5,cross_5x5+25));

    // Open an image with a user-defined structuring element
    Image image4 = opening(sourceImage,se1,1);

    // Display an image in an existing window
    destWindow.setBuffer(image4);
    destWindow.update();
    runMessageLoop("Press space to close the image (3x3 squared with center hole)\n");




    // Create a 3x3 structuring element that has the shape of a square with a hole in the center
    const int square_3x3_hole[] =
      {1,1,1,
       1,0,1,
       1,1,1};
    StructuringElement se2 = StructuringElement(3,3);
    se2.setValues(std::vector<double>(square_3x3_hole,square_3x3_hole+9));

    // Close an image using a user-defined structuring element
    Image image5 = closing(sourceImage,se2,1);

    // Display an image in an existing window
    destWindow.setBuffer(image5);
    destWindow.update();
    runMessageLoop("Press space to quit\n");



  }
  catch(ImpactException& e)
  {
    // Output an error message
    std::cerr << e.what() << std::endl;
    printf("Press a key in the console window to exit\n");
    while(!keyHit());
  }

  return 0;
}
//</matrixdoc>
