/**
\xmlonly
mvIMPACT OCR
\endxmlonly

\page ocr2_page Advanced OCR example (CPP)

\section ocr2_section_purpose Purpose
This example illustrates how to recognize dotted characters from a can.

\section ocr2_section_intro What it does
The example shows the bottom of a can with the expiration date. At the beginning the sample makes some 
processings to make the characters readable. Then it divides the text (including the characters) into 
segments. If the segmentation was successfully, by using an OCR font database the OCR is executed on the image.

\section ocr2_section_description How it works

A class derived from Window is defined. In the constructor, the parent's constructor is called and
the member variable, i.e. the SegmentList object is initialized. Then a simple setter function for
the segment list is defined.

\code
class MyWindow: public Window 
{
private:
	SegmentList* m_list;

public:
	MyWindow() 
		: Window(), m_list(NULL)
	{ }

	MyWindow(const Image &image) 
		: Window(image), m_list(NULL)
	{ }

	MyWindow(const MyWindow& r)
		: Window(r), m_list(r.m_list)
	{ }

	virtual MyWindow* makeShallowCopy() const
	{
		return new MyWindow(*this);
	}

	MyWindow & operator=(const MyWindow & src)
	{
		Window::operator=(src);
		m_list = src.m_list;
		return *this;
	}

	//used to update segment list
	void setSegmentList(SegmentList *list) {
		m_list = list;
		redraw();
	}
\endcode

Afterwards the onDraw method is overwritten to draw the segments from the list into the window.

\code
	//called each time the window will be drawn
	void onDraw (
		DeviceContext *pDc, 
		const Rectangle * ,
		const Rectangle * ) 
	{
		if(m_list) {
			// get the current scroll position of the window
			int	spx = getScrollPosX();
			int spy = getScrollPosY();

			//draw the segments from the segment list
			for (int counter=0; counter<m_list->getSize();counter++) {
				Rectangle rect = m_list->getSegment(counter);
				Point p0 = 
					Point(rect.getLeft() + childXOff - spx, rect.getTop() + childYOff - spy);
				Point p1 = 
					Point(rect.getRight() + childXOff - spx, rect.getTop() + childYOff - spy);
				Point p2 = 
					Point(rect.getRight() + childXOff - spx, rect.getBottom() + childYOff - spy);
				Point p3 = 
					Point(rect.getLeft() + childXOff - spx, rect.getBottom() + childYOff - spy);

				pDc->drawLine(p0,p1,RGBColor(0,255,0));
				pDc->drawLine(p1,p2,RGBColor(0,255,0));
				pDc->drawLine(p2,p3,RGBColor(0,255,0));
				pDc->drawLine(p3,p0,RGBColor(0,255,0));

			}
		}
	} 
\endcode


In the main program the input image is loaded and displayed in a window.

\code
		//load and display image
		Image image = loadImage("can.bmp");
		MyWindow window = MyWindow(image);

\endcode

The dotted characters of the can need a preprocessing. Therefore a child image and a user defined structuring
element representing are defined. After this the child image is eroded and the window is redrawn to display
the result of the preprocessing operation.

\code
		//create child image offset:200/200 w/h:240/160
		Image child = Image(image,childXOff,childYOff,240,160);

		//erosion (only on child area)
		StructuringElement se = StructuringElement(3,3);

		const std::vector<double> values(9, 1.);
		se.setValues(values);

		erode(child,child,structElement);

		//update window
		window.redraw();
\endcode

The OCR font database is loaded and the properties of the segmentation are defined (these
settings can be used to optimize the speed of the complete OCR process). 

\code
		//wait for space key
		runMessageLoop("Press <space> to segment image\n");


		//load OCR font database
		std::string file;
		if (getenv("VL_FILE_DIR") != NULL)
			file = getenv("VL_FILE_DIR") + std::string("pattern.ocr");
		else
			file = std::string("pattern.ocr");

		OcrFontDatabase ocrFontDB = OcrFontDatabase(file,true);

		//set some properties for the segmentation
		ocrFontDB.setClassificationType(ocCorrelation);
		ocrFontDB.setThreshold(127);
		ocrFontDB.setMinCharWidth(15);
		ocrFontDB.setMinCharHeight(25);
		ocrFontDB.setMaxCharWidth(30);
		ocrFontDB.setMaxCharHeight(40);
\endcode

Then a new SegmentList object is created and the image is segmented.

\code
		SegmentList segmentList = SegmentList(256);

		window.setSegmentList(&segmentList);

		//segment the child area
		ocrFontDB.segment(child,segmentList);

		//force window update
		window.redraw();
\endcode

The next step optimizes the segments by splitting, merging and sorting the characters.

\code
		//wait for space bar
		runMessageLoop("Press <space> to split\n");

		segmentList.split(ocrFontDB,child);

		//force window update
		window.redraw();


		//wait for space bar
		runMessageLoop("Press <space> to merge\n");

		segmentList.merge(ocrFontDB,child);

		//force window update
		window.redraw();


		//wait for space bar
		runMessageLoop("Press <space> to sort\n");

		segmentList.sort(ocrFontDB,child);
\endcode

After this the result is printed:

\code
		//wait for space bar
		runMessageLoop("Press <space> to read\n");

		for (int rectCounter=0;rectCounter<segmentList.getSize();rectCounter++) {
			Rectangle rect = segmentList.getSegment(rectCounter);
			OcrResult result = ocrFontDB.readChar(child,rect);
			std::cout << (char)result.getAscii();
		}
		//start new line
		std::cout << std::endl;
\endcode

Finally the message loop is run and after the user has pressed space, the pointer 
in the MyWindow object is set to NULL to make destruction of the objects clean independent of
the order in which the destructors are called.

\code
		//keep demo alive
		runMessageLoop();

		// clean up
		window.setSegmentList(NULL);
\endcode

\section ocr2_section_source Source file

<matrixdoc>
*/

#include "mvIMPACT.h"
#include "mvIMPACT_ocr.h"


#define childXOff 200
#define childYOff 200

class MyWindow: public Window 
{
private:
	SegmentList* m_list;

public:
	MyWindow() 
		: Window(), m_list(NULL)
	{ }

	MyWindow(const Image &image) 
		: Window(image), m_list(NULL)
	{ }

	MyWindow(const MyWindow& r)
		: Window(r), m_list(r.m_list)
	{ }

	virtual MyWindow* makeShallowCopy() const
	{
		return new MyWindow(*this);
	}

	MyWindow & operator=(const MyWindow & src)
	{
		Window::operator=(src);
		m_list = src.m_list;
		return *this;
	}

	//used to update segment list
	void setSegmentList(SegmentList *list) {
		m_list = list;
		redraw();
	}

	//called each time the window will be drawn
	void onDraw (
		DeviceContext *pDc, 
		const mvIMPACT::Rectangle * /*pSourceRect*/,
		const mvIMPACT::Rectangle * /*pDestRect*/) 
	{
		if(m_list) {
			// get the current scroll position of the window
			int	spx = getScrollPosX();
			int spy = getScrollPosY();

			//draw the segments from the segment list
			for (int counter=0; counter<m_list->getSize();counter++) {
				mvIMPACT::Rectangle rect = m_list->getSegment(counter);
				Point p0 = 
					Point(rect.getLeft() + childXOff - spx, rect.getTop() + childYOff - spy);
				Point p1 = 
					Point(rect.getRight() + childXOff - spx, rect.getTop() + childYOff - spy);
				Point p2 = 
					Point(rect.getRight() + childXOff - spx, rect.getBottom() + childYOff - spy);
				Point p3 = 
					Point(rect.getLeft() + childXOff - spx, rect.getBottom() + childYOff - spy);

				pDc->drawLine(p0,p1,RGBColor(0,255,0));
				pDc->drawLine(p1,p2,RGBColor(0,255,0));
				pDc->drawLine(p2,p3,RGBColor(0,255,0));
				pDc->drawLine(p3,p0,RGBColor(0,255,0));

			}
		}
	} 
};


int main(int /*argc*/, char* /*argv*/[])
{
	try
	{
		//load and display image
		Image image = loadImage("can.bmp");
		MyWindow window = MyWindow(image);

		//wait for key
		runMessageLoop("Press <space> to preprocess image\n");

		//create child image offset:200/200 w/h:240/160
		Image child = Image(image,childXOff,childYOff,240,160);

		//erosion (only on child area)
		StructuringElement se = StructuringElement(3,3);

		const std::vector<double> values(9, 1.);
		se.setValues(values);

		erode(child,child,se);

		//update window
		window.redraw();

		//wait for space key
		runMessageLoop("Press <space> to segment image\n");

		//load OCR font database
		std::string file;
		if (getenv("VL_FILE_DIR") != NULL)
			file = getenv("VL_FILE_DIR") + std::string("pattern.ocr");
		else
			file = std::string("pattern.ocr");
	
		
		OcrFontDatabase ocrFontDB = OcrFontDatabase(file,true);

		//set some properties for the segmentation
		ocrFontDB.setClassificationType(ocCorrelation);
		ocrFontDB.setThreshold(127);
		ocrFontDB.setMinCharWidth(15);
		ocrFontDB.setMinCharHeight(25);
		ocrFontDB.setMaxCharWidth(30);
		ocrFontDB.setMaxCharHeight(40);


		SegmentList segmentList = SegmentList(256);

		window.setSegmentList(&segmentList);

		//segment the child area
		ocrFontDB.segment(child,segmentList);

		//force window update
		window.redraw();


		//wait for space bar
		runMessageLoop("Press <space> to split\n");

		segmentList.split(ocrFontDB,child);

		//force window update
		window.redraw();


		//wait for space bar
		runMessageLoop("Press <space> to merge\n");

		segmentList.merge(ocrFontDB,child);

		//force window update
		window.redraw();


		//wait for space bar
		runMessageLoop("Press <space> to sort\n");

		segmentList.sort(ocrFontDB,child);


		//wait for space bar
		runMessageLoop("Press <space> to read\n");

		for (int rectCounter=0;rectCounter<segmentList.getSize();rectCounter++) {
			mvIMPACT::Rectangle rect = segmentList.getSegment(rectCounter);
			OcrResult result = ocrFontDB.readChar(child,rect);
			std::cout << (char)result.getAscii();
		}
		//start new line
		std::cout << std::endl;

		//keep demo alive
		runMessageLoop();

		// clean up
		window.setSegmentList(NULL);
	}
	catch(ImpactException& e)
	{
		std::cerr << e.getErrorOrigin() << " : " << e.getErrorString() << " (" << e.getErrorCodeAsString() << ")" << std::endl;
	}

	return 0;
}

//</matrixdoc>
