/**
\xmlonly
mvIMPACT Base
\endxmlonly

\page presetimg_page Preset image example (CPP)

\section presetimg_section_purpose Purpose
This example shows how to preset and image with various patterns.

\section presetimg_section_intro What it does
This example demonstrates the patterns which can be used to preset an image. 
The image is set to a constant value, a checker pattern, a wedge pattern, a ring pattern 
and random noise.

\section presetimg_section_description How it works

First an empty (uninitialized) image is created and displayed.

\code
image = Image(512, 512);
wnd = Window(image);
\endcode

Then the image is preset with constant values from 0 to 255 with a step width of
16.

\code
		std::cout << "Presetting with constant value (ranging from 0 to 255)" 
			<< std::endl << std::endl;

		int i;
		for (i=0; i<256; i+=16)
		{
			presetConstantImage(image, i);
			wnd.redraw();
			dispatchMessages();
		}
\endcode

Then the image is preset with a checkerboard pattern from 0 to 255 with a step width of
16. The other half of the board is initialized with <code>255-i</code> where \c i is the
value of the first half.

\code
		std::cout << "Presetting with checkerboard pattern (ranging from 0 to 255)" 
			<< std::endl << std::endl;

		for (i=0; i<256; i+=16)
		{
			presetCheckerImage(image, PresetColors(i, 255-i), 
				PresetTransformation(32, 32));
			wnd.redraw();
			dispatchMessages();
		}
\endcode

Then the image is preset with a checkerboard pattern at different roation angles ranging from
0 to 90 degrees at a step with of 6.

\code
		std::cout << "Presetting with checkerboard pattern (rotating)" 
			<< std::endl << std::endl;

		for (i=0; i<360; i+=18)
		{
			presetCheckerImage(image, PresetColors(0, 255), 
				PresetTransformation(32, 32, 256, 256, i*M_PI/360));
			wnd.redraw();
			dispatchMessages();
		}
\endcode

Afterwards the image is preset with a wedge pattern set with values
i between 0 and 256 and a step width of 16 for the first color and
255-i for the second color.

\code
		std::cout << "Presetting with wedge pattern (ranging from 0 to 255)" 
			<< std::endl << std::endl;

		for (i=0; i<256; i+=16)
		{
			presetWedgeImage(image, PresetColors(i, 255-i), 
				PresetTransformation(64, 1));
			wnd.redraw();
			dispatchMessages();
		}
\endcode

Afterwards the image is set with gaussian ring patterns ranging again from
0 to 256 with a step with of 16. 

\code
		std::cout << "Presetting with gaussian ring pattern (ranging from 0 to 255)" 
			<< std::endl << std::endl;

		for (i=0; i<256; i+=16)
		{
			presetRingImage(image, PresetColors(i, 255-i), 
				PresetTransformation(64, 64, 256, 256), pctPresetGauss);
			wnd.redraw();
			dispatchMessages();
		}

\endcode

Then, a noise pattern is used as preset. 

\code
		std::cout << "Presetting with noise pattern" << std::endl << std::endl;

		for (i=0; i<256; i+=16)
		{
			presetNoiseImage(image, PresetColors(0, 255), i, pctPresetGauss);
			wnd.redraw();
			dispatchMessages();
		}
\endcode

Finally the a new color image is created and three child images each consisting of
one band of their parent. These images are then set with three different kinds of
preset values. The red channel is preset with a checker pattern, the green channel
with a wedge pattern and the blue channel with a noise pattern.

\code
		Image color(512, 512, 3);
		wnd.setBuffer(color);
		Image red(color, 0, 0, 0, color.getWidth(), color.getHeight(), 1);
		Image green(color, 0, 0, 1, color.getWidth(), color.getHeight(), 1);
		Image blue(color, 0, 0, 2, color.getWidth(), color.getHeight(), 1);

		std::cout << "Now doing a bit of everything in a color image" << std::endl << std::endl;

		for (i=0; i<360; i+=18)
		{
			presetCheckerImage(red, PresetColors(0, 255), 
				PresetTransformation(32, 32, 256, 256, i*M_PI/360));
			presetWedgeImage(green, PresetColors(0, 255), 
				PresetTransformation(64, 1, 256, 256, (360-i)*M_PI/360));
			presetNoiseImage(blue, PresetColors(0, 255), i, pctPresetGauss);
			wnd.redraw();
			dispatchMessages();
		}
\endcode

Finally the user is requested to press space and the program ends.

\code
		// keep sample alive until console window is closed
		// or space bar is pressed
		runMessageLoop();
\endcode


\section presetimg_section_source Source file

<matrixdoc>
*/

#include "mvIMPACT.h"

#ifndef M_PI
#	define M_PI 3.141592654
#endif // #ifndef M_PI

int main(int /*argc*/, char* /*argv*/[])
{
	try
	{
		std::cout << "This sample shows how to preset an image with various ";
		std::cout << "predefined patterns.";
		std::cout << std::endl << std::endl;

		Image image(512, 512);
		Window wnd(image);

		std::cout << "Presetting with constant value (ranging from 0 to 255)" 
			<< std::endl << std::endl;

		int i;
		for (i=0; i<256; i+=16)
		{
			presetConstantImage(image, i);
			wnd.redraw();
			dispatchMessages();
		}

		std::cout << "Presetting with checkerboard pattern (ranging from 0 to 255)" 
			<< std::endl << std::endl;

		for (i=0; i<256; i+=16)
		{
			presetCheckerImage(image, PresetColors(i, 255-i), 
				PresetTransformation(32, 32));
			wnd.redraw();
			dispatchMessages();
		}

		std::cout << "Presetting with checkerboard pattern (rotating)" 
			<< std::endl << std::endl;

		for (i=0; i<360; i+=18)
		{
			presetCheckerImage(image, PresetColors(0, 255), 
				PresetTransformation(32, 32, 256, 256, i*M_PI/360));
			wnd.redraw();
			dispatchMessages();
		}

		std::cout << "Presetting with wedge pattern (ranging from 0 to 255)" 
			<< std::endl << std::endl;

		for (i=0; i<256; i+=16)
		{
			presetWedgeImage(image, PresetColors(i, 255-i), 
				PresetTransformation(64, 1));
			wnd.redraw();
			dispatchMessages();
		}

		std::cout << "Presetting with gaussian ring pattern (ranging from 0 to 255)" 
			<< std::endl << std::endl;

		for (i=0; i<256; i+=16)
		{
			presetRingImage(image, PresetColors(i, 255-i), 
				PresetTransformation(64, 64, 256, 256), pctPresetGauss);
			wnd.redraw();
			dispatchMessages();
		}

		std::cout << "Presetting with noise pattern" << std::endl << std::endl;

		for (i=0; i<256; i+=16)
		{
			presetNoiseImage(image, PresetColors(0, 255), i, pctPresetGauss);
			wnd.redraw();
			dispatchMessages();
		}

		Image color(512, 512, 3);
		wnd.setBuffer(color);
		Image red(color, 0, 0, 0, color.getWidth(), color.getHeight(), 1);
		Image green(color, 0, 0, 1, color.getWidth(), color.getHeight(), 1);
		Image blue(color, 0, 0, 2, color.getWidth(), color.getHeight(), 1);

		std::cout << "Now doing a bit of everything in a color image" << std::endl << std::endl;

		for (i=0; i<360; i+=18)
		{
			presetCheckerImage(red, PresetColors(0, 255), 
				PresetTransformation(32, 32, 256, 256, i*M_PI/360));
			presetWedgeImage(green, PresetColors(0, 255), 
				PresetTransformation(64, 1, 256, 256, (360-i)*M_PI/360));
			presetNoiseImage(blue, PresetColors(0, 255), i, pctPresetGauss);
			wnd.redraw();
			dispatchMessages();
		}

		// keep sample alive until console window is closed
		// or space bar is pressed
		runMessageLoop();
	}
	catch(ImpactException& e)
	{
		std::cerr << e.getErrorString() << std::endl;
	}

	return 0;
}

//</matrixdoc>
